#
# Copyright Red Hat, Inc. 2012
#
# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.
#

"""
Unit tests that do readonly functional tests against real bugzilla instances.
"""
import pytest

import bugzilla
import tests


REDHAT_URL = (tests.CLICONFIG.REDHAT_URL or
    "https://bugzilla.redhat.com")


def _open_bz(url, bzclass=None, **kwargs):
    if "use_creds" not in kwargs:
        kwargs["use_creds"] = False
    return tests.utils.open_functional_bz(bzclass or bugzilla.Bugzilla,
            url, kwargs)


def _check(out, mincount, expectstr):
    # Since we are running these tests against bugzilla instances in
    # the wild, we can't depend on certain data like product lists
    # remaining static. Use lax sanity checks in this case
    if mincount is not None:
        assert len(out.splitlines()) >= mincount
    assert expectstr in out


def _test_version(bz, bzversion):
    assert bz.bz_ver_major == bzversion[0]
    assert bz.bz_ver_minor == bzversion[1]


def test_bugzilla_override():
    class MyBugzilla(bugzilla.Bugzilla):
        pass

    bz = _open_bz("bugzilla.redhat.com", bzclass=MyBugzilla)
    assert bz.__class__ is MyBugzilla
    assert bz._is_redhat_bugzilla is True  # pylint: disable=protected-access


def test_rest_xmlrpc_detection():
    # The default: use XMLRPC
    bz = _open_bz("bugzilla.redhat.com")
    assert bz.is_xmlrpc()
    assert "/xmlrpc.cgi" in bz.url
    assert bz.__class__ is bugzilla.RHBugzilla

    # See /rest in the URL, so use REST
    bz = _open_bz("bugzilla.redhat.com/rest")
    assert bz.is_rest()
    with pytest.raises(bugzilla.BugzillaError) as e:
        dummy = bz._proxy  # pylint: disable=protected-access
    assert "raw XMLRPC access is not provided" in str(e)

    # See /xmlrpc.cgi in the URL, so use XMLRPC
    bz = _open_bz("bugzilla.redhat.com/xmlrpc.cgi")
    assert bz.is_xmlrpc()
    assert bz._proxy  # pylint: disable=protected-access


def test_apikey_error_scraping():
    # Ensure the API key does not leak into any requests exceptions
    fakekey = "FOOBARMYKEY"
    with pytest.raises(Exception) as e:
        _open_bz("https://httpstat.us/502&foo",
                force_xmlrpc=True, api_key=fakekey)
    assert "Client Error" in str(e.value)
    assert fakekey not in str(e.value)

    with pytest.raises(Exception) as e:
        _open_bz("https://httpstat.us/502&foo",
                force_rest=True, api_key=fakekey)
    assert "Client Error" in str(e.value)
    assert fakekey not in str(e.value)


def test_xmlrpc_bad_url():
    with pytest.raises(bugzilla.BugzillaError) as e:
        _open_bz("https://example.com/#xmlrpc")
    assert "URL may not be an XMLRPC URL" in str(e)


###################
# mozilla testing #
###################

def test_mozilla(backends):
    url = "bugzilla.mozilla.org"
    bz = _open_bz(url, **backends)

    # bugzilla.mozilla.org returns version values in YYYY-MM-DD
    # format, so just try to confirm that
    assert bz.__class__ == bugzilla.Bugzilla
    assert bz.bz_ver_major >= 2016
    assert bz.bz_ver_minor in range(1, 13)


##################
# gentoo testing #
##################

def test_gentoo(backends):
    url = "bugs.gentoo.org"
    bzversion = (5, 0)
    bz = _open_bz(url, **backends)
    _test_version(bz, bzversion)

    # This is a bugzilla 5.0 instance, which supports URL queries now
    query_url = ("https://bugs.gentoo.org/buglist.cgi?"
        "component=[CS]&product=Doc%20Translations"
        "&query_format=advanced&resolution=FIXED")
    ret = bz.query(bz.url_to_query(query_url))
    assert len(ret) > 0


##################
# redhat testing #
##################


def testInfoProducts(run_cli, backends):
    bz = _open_bz(REDHAT_URL, **backends)

    out = run_cli("bugzilla info --products", bz)
    _check(out, 123, "Virtualization Tools")


def testInfoComps(run_cli, backends):
    bz = _open_bz(REDHAT_URL, **backends)

    out = run_cli("bugzilla info --components 'Virtualization Tools'", bz)
    _check(out, 8, "virtinst")


def testInfoVers(run_cli, backends):
    bz = _open_bz(REDHAT_URL, **backends)

    out = run_cli("bugzilla info --versions Fedora", bz)
    _check(out, 17, "rawhide")


def testInfoCompOwners(run_cli, backends):
    bz = _open_bz(REDHAT_URL, **backends)

    out = run_cli("bugzilla info "
            "--component_owners 'Virtualization Tools'", bz)
    _check(out, None, "libvirt: Libvirt Maintainers")


def testQuery(run_cli, backends):
    bz = _open_bz(REDHAT_URL, **backends)

    args = "--product Fedora --component python-bugzilla --version 14"
    cli = "bugzilla query %s --bug_status CLOSED" % args
    mincount = 4
    expectbug = "621030"
    out = run_cli(cli, bz)

    assert len(out.splitlines()) >= mincount
    assert bool([l1 for l1 in out.splitlines() if
                 l1.startswith("#" + expectbug)])

    # Check --ids output option
    out2 = run_cli(cli + " --ids", bz)
    assert len(out.splitlines()) == len(out2.splitlines())
    assert bool([l2 for l2 in out2.splitlines() if
                 l2 == expectbug])


def testQueryFull(run_cli, backends):
    bz = _open_bz(REDHAT_URL, **backends)

    bugid = "621601"
    out = run_cli("bugzilla query --full --bug_id %s" % bugid, bz)
    _check(out, 60, "end-of-life (EOL)")


def testQueryRaw(run_cli, backends):
    bz = _open_bz(REDHAT_URL, **backends)

    bugid = "307471"
    out = run_cli("bugzilla query --raw --bug_id %s" % bugid, bz)
    _check(out, 70, "ATTRIBUTE[whiteboard]:  bzcl34nup")


def testQueryOneline(run_cli, backends):
    bz = _open_bz(REDHAT_URL, **backends)

    bugid = "785016"
    out = run_cli("bugzilla query --oneline --bug_id %s" % bugid, bz)
    assert len(out.splitlines()) == 1
    assert out.splitlines()[0].startswith("#%s" % bugid)
    assert "[---] fedora-review+,fedora-cvs+" in out

    bugid = "720784"
    out = run_cli("bugzilla query --oneline --bug_id %s" % bugid, bz)
    assert len(out.splitlines()) == 1
    assert out.splitlines()[0].startswith("#%s" % bugid)
    assert " CVE-2011-2527" in out


def testQueryExtra(run_cli, backends):
    bz = _open_bz(REDHAT_URL, **backends)

    bugid = "307471"
    out = run_cli("bugzilla query --extra --bug_id %s" % bugid, bz)
    assert ("#%s" % bugid) in out
    assert " +Status Whiteboard:  bzcl34nup" in out


def testQueryFormat(run_cli, backends):
    bz = _open_bz(REDHAT_URL, **backends)

    args = ("--bug_id 307471 --outputformat=\"id=%{bug_id} "
        "sw=%{whiteboard:status} needinfo=%{flag:needinfo} "
        "sum=%{summary}\"")
    out = run_cli("bugzilla query %s" % args, bz)
    assert "id=307471 sw= bzcl34nup needinfo= " in out

    args = ("--bug_id 785016 --outputformat=\"id=%{bug_id} "
        "sw=%{whiteboard:status} flag=%{flag:fedora-review} "
        "sum=%{summary}\"")
    out = run_cli("bugzilla query %s" % args, bz)
    assert "id=785016 sw= flag=+" in out

    # Unicode in this bug's summary
    args = "--bug_id 522796 --outputformat \"%{summary}\""
    out = run_cli("bugzilla query %s" % args, bz)
    assert "V34 — system" in out


def testQueryURL(run_cli, backends):
    bz = _open_bz(REDHAT_URL, **backends)

    qurl = ("/buglist.cgi?f1=creation_ts"
        "&list_id=973582&o1=greaterthaneq&classification=Fedora&"
        "o2=lessthaneq&query_format=advanced&f2=creation_ts"
        "&v1=2010-01-01&component=python-bugzilla&v2=2010-06-01"
        "&product=Fedora")

    url = REDHAT_URL
    if "/xmlrpc.cgi" in url:
        url = url.replace("/xmlrpc.cgi", qurl)
    else:
        url += qurl
    out = run_cli("bugzilla query --from-url \"%s\"" % url, bz)
    _check(out, 10, "#553878 CLOSED")


def testQueryFixedIn(run_cli, backends):
    bz = _open_bz(REDHAT_URL, **backends)

    out = run_cli("bugzilla query --fixed_in anaconda-15.29-1", bz)
    assert len(out.splitlines()) == 4
    assert "#629311 CLOSED" in out


def testQueryExtrafieldPool(run_cli, backends):
    # rhbz has an agile 'pool' extension but doesn't return the field
    # by default. Check that '-extrafield pool' returns it for --json output
    bz = _open_bz(REDHAT_URL, **backends)

    out1 = run_cli("bugzilla query --id 1717616 --json", bz)
    out2 = run_cli("bugzilla query --id 1717616 --json --extrafield pool", bz)
    assert "current_sprint_id" not in out1
    assert "current_sprint_id" in out2


def testComponentsDetails(backends):
    """
    Fresh call to getcomponentsdetails should properly refresh
    """
    bz = _open_bz(REDHAT_URL, **backends)

    assert bool(bz.getcomponentsdetails("Red Hat Developer Toolset"))


def testGetBugAlias(backends):
    """
    getbug() works if passed an alias
    """
    bz = _open_bz(REDHAT_URL, **backends)

    bug = bz.getbug("CVE-2011-2527")
    assert bug.bug_id == 720773


def testQuerySubComponent(run_cli, backends):
    bz = _open_bz(REDHAT_URL, **backends)

    # Test special error wrappers in bugzilla/_cli.py
    out = run_cli("bugzilla query --product 'Red Hat Enterprise Linux 7' "
        "--component lvm2 --sub-component 'Thin Provisioning'", bz)
    assert len(out.splitlines()) >= 3
    assert "#1060931 " in out


def testBugFields(backends):
    bz = _open_bz(REDHAT_URL, **backends)

    fields = bz.getbugfields(names=["product"])[:]
    assert fields == ["product"]
    bz.getbugfields(names=["product", "bug_status"], force_refresh=True)
    assert set(bz.bugfields) == set(["product", "bug_status"])


def testProductGetMisc(backends):
    bz = _open_bz(REDHAT_URL, **backends)

    assert bz.product_get(ptype="enterable", include_fields=["id"])
    assert bz.product_get(ptype="selectable", include_fields=["name"])


def testBugAutoRefresh(backends):
    bz = _open_bz(REDHAT_URL, **backends)

    bz.bug_autorefresh = True

    bug = bz.query(bz.build_query(bug_id=720773,
        include_fields=["summary"]))[0]
    assert hasattr(bug, "component")
    assert bool(bug.component)

    bz.bug_autorefresh = False

    bug = bz.query(bz.build_query(bug_id=720773,
        include_fields=["summary"]))[0]
    assert not hasattr(bug, "component")
    try:
        assert bool(bug.component)
    except Exception as e:
        assert "adjust your include_fields" in str(e)


def testExtraFields(backends):
    bz = _open_bz(REDHAT_URL, **backends)

    # Check default extra_fields will pull in comments
    bug = bz.getbug(720773, exclude_fields=["product"])
    assert "comments" in dir(bug)
    assert "product" not in dir(bug)

    # Ensure that include_fields overrides default extra_fields
    bug = bz.getbug(720773, include_fields=["summary"])
    assert "summary" in dir(bug)
    assert "comments" not in dir(bug)


def testExternalBugsOutput(run_cli, backends):
    bz = _open_bz(REDHAT_URL, **backends)

    out = run_cli('bugzilla query --bug_id 989253 '
        '--outputformat="%{external_bugs}"', bz)
    assert "bugzilla.gnome.org/show_bug.cgi?id=703421" in out
    assert "External bug: https://bugs.launchpad.net/bugs/1203576" in out


def testActiveComps(run_cli, backends):
    bz = _open_bz(REDHAT_URL, **backends)

    out = run_cli("bugzilla info --components 'Virtualization Tools' "
            "--active-components", bz)
    assert "virtinst" not in out
    out = run_cli("bugzilla info --component_owners 'Virtualization Tools' "
            "--active-components", bz)
    assert "virtinst" not in out


def testFaults(run_cli, backends):
    bz = _open_bz(REDHAT_URL, **backends)

    # Test special error wrappers in bugzilla/_cli.py
    out = run_cli("bugzilla query --field=IDONTEXIST=FOO", bz,
            expectfail=True)
    assert "Server error:" in out

    out = run_cli("bugzilla "
        "--bugzilla https://example.com/xmlrpc.cgi "
        "query --field=IDONTEXIST=FOO", None, expectfail=True)
    assert "Connection lost/failed" in out

    out = run_cli("bugzilla "
        "--bugzilla https://expired.badssl.com/ "
        "query --bug_id 1234", None, expectfail=True)
    assert "trust the remote server" in out
    assert "--nosslverify" in out


def test_login_stubs(backends):
    bz = _open_bz(REDHAT_URL, **backends)

    # Failed login, verifies our backends are calling the correct API
    with pytest.raises(bugzilla.BugzillaError) as e:
        bz.login("foo", "bar")
    assert "Login failed" in str(e)

    # Works fine when not logged in
    bz.logout()


def test_redhat_version(backends):
    bzversion = (5, 0)
    bz = _open_bz(REDHAT_URL, **backends)

    if not tests.CLICONFIG.REDHAT_URL:
        _test_version(bz, bzversion)



#
# Copyright Red Hat, Inc. 2012
#
# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.
#

"""
Test miscellaneous CLI bits to get build out our code coverage
"""

import base64
import datetime
import json
import xmlrpc.client

import pytest
import requests

import bugzilla

import tests
import tests.mockbackend


def testHelp(run_cli):
    out = run_cli("bugzilla --help", None)
    assert len(out.splitlines()) > 18


def testCmdHelp(run_cli):
    out = run_cli("bugzilla query --help", None)
    assert len(out.splitlines()) > 40


def testVersion(run_cli):
    out = run_cli("bugzilla --version", None)
    assert out.strip() == bugzilla.__version__


def testCookiefileDeprecated(run_cli):
    with pytest.raises(TypeError) as e:
        run_cli("bugzilla --cookiefile foobar login",
                None, expectfail=True)
    assert "cookiefile is deprecated" in str(e)


def testPositionalArgs(run_cli):
    # Make sure cli correctly rejects ambiguous positional args
    out = run_cli("bugzilla login --xbadarg foo",
            None, expectfail=True)
    assert "unrecognized arguments: --xbadarg" in out

    out = run_cli("bugzilla modify 123456 --foobar --status NEW",
            None, expectfail=True)
    assert "unrecognized arguments: --foobar" in out


def testDebug(run_cli):
    # Coverage testing for debug options
    run_cli("bugzilla --bugzilla https:///BADURI --verbose login",
            None, expectfail=True)
    run_cli("bugzilla --bugzilla https:///BADURI --debug login",
            None, expectfail=True)


def testExceptions(run_cli):
    """
    Test exception handling around main()
    """
    fakebz = tests.mockbackend.make_bz(
        bug_search_args=None,
        bug_search_return=KeyboardInterrupt())
    out = run_cli("bugzilla query --bug_id 1", fakebz, expectfail=True)
    assert "user request" in out

    fakebz = tests.mockbackend.make_bz(
        bug_search_args=None,
        bug_search_return=bugzilla.BugzillaError("foo"))
    out = run_cli("bugzilla query --bug_id 1", fakebz, expectfail=True)
    assert "Server error:" in out

    fakebz = tests.mockbackend.make_bz(
        bug_search_args=None,
        bug_search_return=requests.exceptions.SSLError())
    out = run_cli("bugzilla query --bug_id 1", fakebz, expectfail=True)
    assert "trust the remote server" in out

    fakebz = tests.mockbackend.make_bz(
        bug_search_args=None,
        bug_search_return=requests.exceptions.ConnectionError())
    out = run_cli("bugzilla query --bug_id 1", fakebz, expectfail=True)
    assert "Connection lost" in out


def testManualURL(run_cli):
    """
    Test passing a manual URL, to hit those non-testsuite code paths
    """
    try:
        cmd = "bugzilla --bztype foobar "
        cmd += "--bugzilla https:///FAKEURL query --bug_id 1"
        run_cli(cmd, None)
    except Exception as e:
        assert "No host supplied" in str(e)


def test_json_xmlrpc(run_cli):
    # Test --json output with XMLRPC type conversion
    cmd = "bugzilla query --json --id 1165434"

    timestr = '20181209T19:12:12'
    dateobj = datetime.datetime.strptime(timestr, '%Y%m%dT%H:%M:%S')

    attachfile = tests.utils.tests_path("data/bz-attach-get1.txt")
    attachdata = open(attachfile, "rb").read()

    bugid = 1165434
    data = {"bugs": [{
        'id': bugid,
        'timetest': xmlrpc.client.DateTime(dateobj),
        'binarytest': xmlrpc.client.Binary(attachdata),
    }]}

    fakebz = tests.mockbackend.make_bz(
        bug_search_args=None,
        bug_search_return={"bugs": [{"id": bugid}]},
        bug_get_args=None,
        bug_get_return=data)
    out = run_cli(cmd, fakebz)
    tests.utils.diff_compare(tests.utils.sanitize_json(out),
        "data/clioutput/test_json_xmlrpc.txt")

    retdata = json.loads(out)["bugs"][0]
    assert (base64.b64decode(retdata["binarytest"]) ==
            attachdata)
    assert retdata["timetest"] == dateobj.isoformat() + "Z"


    # Test an error case, json converter can't handle Exception class
    data["bugs"][0]["foo"] = Exception("foo")
    fakebz = tests.mockbackend.make_bz(
        bug_search_args=None,
        bug_search_return={"bugs": [{"id": bugid}]},
        bug_get_args=None,
        bug_get_return=data)
    with pytest.raises(RuntimeError):
        run_cli(cmd, fakebz, expectfail=True)



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

import os

import tests
import tests.mockbackend
import tests.utils


##################################
# 'bugzilla attach' mock testing #
##################################

def test_attach(run_cli):
    attachfile = os.path.dirname(__file__) + "/data/bz-attach-get1.txt"
    attachcontent = open(attachfile).read()

    # Hit error when no ID specified
    fakebz = tests.mockbackend.make_bz()
    out = run_cli("bugzilla attach", fakebz, expectfail=True)
    assert "ID must be specified" in out

    # Hit error when using tty and no --file specified
    out = run_cli("bugzilla attach 123456", fakebz, expectfail=True)
    assert "--file must be specified" in out

    # Hit error when using stdin, but no --desc
    out = run_cli("bugzilla attach 123456", fakebz, expectfail=True,
            stdin=attachcontent)
    assert "--description must be specified" in out

    # Basic CLI attach
    cmd = "bugzilla attach 123456 --file=%s " % attachfile
    cmd += "--type text/x-patch --private "
    cmd += "--comment 'some comment to go with it'"
    fakebz = tests.mockbackend.make_bz(
        bug_attachment_create_args="data/mockargs/test_attach1.txt",
        bug_attachment_create_return={'ids': [1557949]})
    out = run_cli(cmd, fakebz)
    assert "Created attachment 1557949 on bug 123456" in out

    # Attach from stdin
    cmd = "bugzilla attach 123456 --file=fake-file-name.txt "
    cmd += "--description 'Some attachment description' "
    fakebz = tests.mockbackend.make_bz(
        bug_attachment_create_args="data/mockargs/test_attach2.txt",
        bug_attachment_create_return={'ids': [1557949]})
    out = run_cli(cmd, fakebz, stdin=attachcontent)
    assert "Created attachment 1557949 on bug 123456" in out


def _test_attach_get(run_cli):
    # Hit error when using ids with --get*
    fakebz = tests.mockbackend.make_bz()
    out = run_cli("bugzilla attach 123456 --getall 123456",
            fakebz, expectfail=True)
    assert "not used for" in out

    # Basic --get ATTID usage
    filename = "Klíč memorial test file.txt"
    cmd = "bugzilla attach --get 112233"
    fakebz = tests.mockbackend.make_bz(
        bug_attachment_get_args="data/mockargs/test_attach_get1.txt",
        bug_attachment_get_return="data/mockreturn/test_attach_get1.txt")
    out = run_cli(cmd, fakebz)
    assert filename in out

    # Basic --getall with --ignore-obsolete
    cmd = "bugzilla attach --getall 663674 --ignore-obsolete"
    fakebz = tests.mockbackend.make_bz(
        bug_attachment_get_all_args="data/mockargs/test_attach_get2.txt",
        bug_attachment_get_all_return="data/mockreturn/test_attach_get2.txt")
    out = run_cli(cmd, fakebz)

    os.system("ls %s" % os.getcwd())
    filename += ".1"
    assert filename in out
    assert "bugzilla-filename" in out


def test_attach_get(run_cli):
    import tempfile
    import shutil
    tmpdir = tempfile.mkdtemp(dir=os.getcwd())
    origcwd = os.getcwd()
    os.chdir(tmpdir)
    try:
        _test_attach_get(run_cli)
    finally:
        os.chdir(origcwd)
        shutil.rmtree(tmpdir)



#!/usr/bin/env python
#
# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

# query.py: Perform a few varieties of queries

import time

import bugzilla

# public test instance of bugzilla.redhat.com. It's okay to make changes
URL = "bugzilla.stage.redhat.com"

bzapi = bugzilla.Bugzilla(URL)


# build_query is a helper function that handles some bugzilla version
# incompatibility issues. All it does is return a properly formatted
# dict(), and provide friendly parameter names. The param names map
# to those accepted by Bugzilla Bug.search:
# https://bugzilla.readthedocs.io/en/latest/api/core/v1/bug.html#search-bugs
query = bzapi.build_query(
    product="Fedora",
    component="python-bugzilla")

# Since 'query' is just a dict, you could set your own parameters too, like
# if your bugzilla had a custom field. This will set 'status' for example,
# but for common opts it's better to use build_query
query["status"] = "CLOSED"

# query() is what actually performs the query. it's a wrapper around Bug.search
t1 = time.time()
bugs = bzapi.query(query)
t2 = time.time()
print("Found %d bugs with our query" % len(bugs))
print("Query processing time: %s" % (t2 - t1))


# Depending on the size of your query, you can massively speed things up
# by telling bugzilla to only return the fields you care about, since a
# large chunk of the return time is transmitting the extra bug data. You
# tweak this with include_fields:
# https://wiki.mozilla.org/Bugzilla:BzAPI#Field_Control
# Bugzilla will only return those fields listed in include_fields.
query = bzapi.build_query(
    product="Fedora",
    component="python-bugzilla",
    include_fields=["id", "summary"])
t1 = time.time()
bugs = bzapi.query(query)
t2 = time.time()
print("Quicker query processing time: %s" % (t2 - t1))


# bugzilla.redhat.com, and bugzilla >= 5.0 support queries using the same
# format as is used for 'advanced' search URLs via the Web UI. For example,
# I go to bugzilla.stage.redhat.com -> Search -> Advanced Search, select
#   Classification=Fedora
#   Product=Fedora
#   Component=python-bugzilla
#   Unselect all bug statuses (so, all status values)
#   Under Custom Search
#      Creation date -- is less than or equal to -- 2010-01-01
#
# Run that, copy the URL and bring it here, pass it to url_to_query to
# convert it to a dict(), and query as usual
query = bzapi.url_to_query("https://bugzilla.stage.redhat.com/"
    "buglist.cgi?classification=Fedora&component=python-bugzilla&"
    "f1=creation_ts&o1=lessthaneq&order=Importance&product=Fedora&"
    "query_format=advanced&v1=2010-01-01")
query["include_fields"] = ["id", "summary"]
bugs = bzapi.query(query)
print("The URL query returned 22 bugs... "
      "I know that without even checking because it shouldn't change!... "
      "(count is %d)" % len(bugs))


# One note about querying... you can get subtley different results if
# you are not logged in. Depending on your bugzilla setup it may not matter,
# but if you are dealing with private bugs, check bzapi.logged_in setting
# to ensure your cached credentials are up to date. See update.py for
# an example usage



# python-bugzilla - a Python interface to bugzilla using xmlrpclib.
#
# Copyright (C) 2007, 2008 Red Hat Inc.
# Author: Will Woods <wwoods@redhat.com>
#
# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

from .apiversion import version, __version__
from .base import Bugzilla
from .exceptions import BugzillaError
from .oldclasses import (Bugzilla3, Bugzilla32, Bugzilla34, Bugzilla36,
        Bugzilla4, Bugzilla42, Bugzilla44,
        NovellBugzilla, RHBugzilla, RHBugzilla3, RHBugzilla4)


# This is the public API. If you are explicitly instantiating any other
# class, using some function, or poking into internal files, don't complain
# if things break on you.
__all__ = [
    "Bugzilla3", "Bugzilla32", "Bugzilla34", "Bugzilla36",
    "Bugzilla4", "Bugzilla42", "Bugzilla44",
    "NovellBugzilla",
    "RHBugzilla3", "RHBugzilla4", "RHBugzilla",
    'BugzillaError',
    'Bugzilla', "version",
]


# Clear all other locals() from the public API
for __sym in locals().copy():
    if __sym.startswith("__") or __sym in __all__:
        continue
    locals().pop(__sym)
locals().pop("__sym")



#
# Copyright Red Hat, Inc. 2012
#
# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.
#

"""
Test miscellaneous API bits
"""

import tests
import tests.mockbackend


def test_api_groups():
    # Basic API testing of the users APIs
    group_ret = {"groups": [{
        "membership": [
            {"real_name": "Bugzilla User",
            "can_login": 1,
            "name": "user@bugzilla.org",
            "login_denied_text": "",
            "id": 85,
            "email_enabled": 1,
            "email": "user@bugzilla.org"},
            {"real_name": "Bugzilla User2",
            "can_login": 0,
            "name": "user2@bugzilla.org",
            "login_denied_text": "",
            "id": 77,
            "email_enabled": 0,
            "email": "user2@bugzilla.org"},
        ],
        "is_active": 1,
        "description": "Test Group",
        "user_regexp": "",
        "is_bug_group": 1,
        "name": "TestGroup",
        "id": 9
    }]}

    fakebz = tests.mockbackend.make_bz(
        group_get_args="data/mockargs/test_api_groups_get1.txt",
        group_get_return=group_ret)

    # getgroups testing
    groupobj = fakebz.getgroups("TestGroups")[0]
    assert groupobj.groupid == 9
    assert groupobj.member_emails == [
        "user2@bugzilla.org", "user@bugzilla.org"]
    assert groupobj.name == "TestGroup"

    # getgroup testing
    fakebz = tests.mockbackend.make_bz(
        group_get_args="data/mockargs/test_api_groups_get2.txt",
        group_get_return=group_ret)
    groupobj = fakebz.getgroup("TestGroup", membership=True)
    groupobj.membership = []
    assert groupobj.members() == group_ret["groups"][0]["membership"]



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

from .base import Bugzilla

# These are old compat classes. Nothing new should be added here,
# and these should not be altered


class Bugzilla3(Bugzilla):
    pass


class Bugzilla32(Bugzilla):
    pass


class Bugzilla34(Bugzilla):
    pass


class Bugzilla36(Bugzilla):
    pass


class Bugzilla4(Bugzilla):
    pass


class Bugzilla42(Bugzilla):
    pass


class Bugzilla44(Bugzilla):
    pass


class NovellBugzilla(Bugzilla):
    pass


class RHBugzilla(Bugzilla):
    """
    Helper class for historical bugzilla.redhat.com back compat

    Historically this class used many more non-upstream methods, but
    in 2012 RH started dropping most of its custom bits. By that time,
    upstream BZ had most of the important functionality.

    Much of the remaining code here is just trying to keep things operating
    in python-bugzilla back compatible manner.

    This class was written using bugzilla.redhat.com's API docs:
    https://bugzilla.redhat.com/docs/en/html/api/
    """
    _is_redhat_bugzilla = True


class RHBugzilla3(RHBugzilla):
    pass


class RHBugzilla4(RHBugzilla):
    pass



#!/usr/bin/env python
#
# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

# bug_autorefresh.py: Show what bug_autorefresh is all about, and explain
#   how to handle the default change via python-bugzilla in 2016

import bugzilla

# public test instance of bugzilla.redhat.com. It's okay to make changes
URL = "bugzilla.stage.redhat.com"
bzapi = bugzilla.Bugzilla(URL)

# The Bugzilla.bug_autorefresh setting controls whether bugs will
# automatically go out and try to update their cached contents when code
# tries to access a bug attribute that isn't already cached.
#
# Note this is likely only relevant if some part of your code is using
# include_fields, or exclude_fields, or you are depending on access
# to bugzilla.redhat.com 'extra_fields' type data like 'attachments'
# without explicitly asking the API for them. If you aren't using any
# of those bits, you can ignore this.
#
# Though if you aren't using include_fields and you are running regular
# queries in a script, check examples/query.py for a simple usecase that
# shows how much include_fields usage can speed up your scripts.

# The default as of mid 2016 is bug_autorefresh=off, so set it True here
# to demonstrate
bzapi.bug_autorefresh = True
bug = bzapi.getbug(427301, include_fields=["id", "summary"])

# The limited include_fields here means that only "id" and "summary" fields
# of the bug are cached in the bug object. What happens when we try to
# get component for example?
print("Bug component=%s" % bug.component)

# Because bug_autorefresh is True, the bug object basically did a
# a bug.refresh() for us, grabbed all its data, and now the component field
# is there. Let's try it again, but this time without bug_autorefresh
bzapi.bug_autorefresh = False
bug = bzapi.getbug(427301, include_fields=["id", "summary"])
try:
    print("Shouldn't see this! bug component=%s" % bug.component)
except AttributeError:
    print("With bug_autorefresh=False, we received AttributeError as expected")

# Why does this matter? Some scripts are implicitly depending on this
# auto-refresh behavior, because their include_fields specification doesn't
# cover all attributes they actually use. Your script will work, sure, but
# it's likely doing many more API calls than needed, possibly 1 per bug.
# So if after upgrading python-bugzilla you start hitting issues, the
# recommendation is to fix your include_fields.



#
# Copyright Red Hat, Inc. 2014
#
# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.
#

"""
Unit tests for testing some bug.py magic
"""

import io
import pickle

import pytest

import tests
import tests.mockbackend
import tests.utils

from bugzilla.bug import Bug


rhbz = tests.mockbackend.make_bz(version="4.4.0", rhbz=True)


def testBasic():
    data = {
        "bug_id": 123456,
        "status": "NEW",
        "assigned_to": "foo@bar.com",
        "component": "foo",
        "product": "bar",
        "short_desc": "some short desc",
        "cf_fixed_in": "nope",
        "fixed_in": "1.2.3.4",
        "devel_whiteboard": "some status value",
    }

    bug = Bug(bugzilla=rhbz, dict=data)

    def _assert_bug():
        assert hasattr(bug, "component") is True
        assert getattr(bug, "components") == ["foo"]
        assert getattr(bug, "product") == "bar"
        assert hasattr(bug, "short_desc") is True
        assert getattr(bug, "summary") == "some short desc"
        assert bool(getattr(bug, "cf_fixed_in")) is True
        assert getattr(bug, "fixed_in") == "1.2.3.4"
        assert bool(getattr(bug, "cf_devel_whiteboard")) is True
        assert getattr(bug, "devel_whiteboard") == "some status value"

    _assert_bug()

    assert str(bug) == "#123456 NEW        - foo@bar.com - some short desc"
    assert repr(bug).startswith("<Bug #123456")

    # This triggers some code in __getattr__
    dir(bug)

    # Test special pickle support
    fd = io.BytesIO()
    pickle.dump(bug, fd)
    fd.seek(0)
    bug = pickle.load(fd)
    assert getattr(bug, "bugzilla") is None
    assert str(bug)
    assert repr(bug)
    bug.bugzilla = rhbz
    _assert_bug()


def testBugNoID():
    try:
        Bug(bugzilla=rhbz, dict={"component": "foo"})
        raise AssertionError("Expected lack of ID failure.")
    except TypeError:
        pass


def test_api_getbugs():
    fakebz = tests.mockbackend.make_bz(
        bug_get_args="data/mockargs/test_api_getbugs1.txt",
        bug_get_return="data/mockreturn/test_query_cve_getbug.txt")

    fakebz.bug_autorefresh = True
    bug = fakebz.getbug("CVE-1234-5678", exclude_fields="foo")
    assert bug.alias == ["CVE-1234-5678"]
    assert bug.autorefresh is True

    fakebz = tests.mockbackend.make_bz(
        bug_get_args="data/mockargs/test_api_getbugs2.txt",
        bug_get_return={"bugs": [{}, {}]})
    assert fakebz.getbugs(["123456", "CVE-1234-FAKE"]) == []


def test_bug_getattr():
    fakebz = tests.mockbackend.make_bz(
        bug_get_args=None,
        bug_get_return="data/mockreturn/test_getbug_rhel.txt")
    bug = fakebz.getbug(1165434)

    with pytest.raises(AttributeError):
        # Hits a specific codepath in Bug.__getattr__
        dummy = bug.__baditem__

    bug.autorefresh = True
    summary = bug.summary
    del(bug.__dict__["summary"])
    # Trigger autorefresh
    assert bug.summary == summary


def test_bug_apis():
    def _get_fake_bug(apiname):
        update_args = "data/mockargs/test_bug_apis_%s_update.txt" % apiname
        fakebz = tests.mockbackend.make_bz(rhbz=True,
            bug_get_args=None,
            bug_get_return="data/mockreturn/test_getbug_rhel.txt",
            bug_update_args=update_args,
            bug_update_return={})
        return fakebz.getbug(1165434)

    # bug.setstatus, wrapper for update_bugs
    bug = _get_fake_bug("setstatus")
    bug.setstatus("POST", "foocomment", private=True)

    # bug.close, wrapper for update_bugs
    bug = _get_fake_bug("close")
    bug.close("UPSTREAM", dupeid=123456, comment="foocomment2",
            isprivate=False, fixedin="1.2.3.4.5")

    # bug.setassignee, wrapper for update_bugs
    bug = _get_fake_bug("setassignee")
    bug.setassignee(
        assigned_to="foo@example.com", qa_contact="bar@example.com",
        comment="foocomment")
    with pytest.raises(ValueError):
        # Hits a validation path
        bug.setassignee()

    # bug.addcc test
    bug = _get_fake_bug("addcc")
    bug.addcc("foo2@example.com", comment="foocomment")

    # bug.deletecc test
    bug = _get_fake_bug("deletecc")
    bug.deletecc("foo2@example.com", comment="foocomment")

    # bug.addcomment test
    bug = _get_fake_bug("addcomment")
    bug.addcomment("test comment", private=True)

    # bug.updateflags test
    bug = _get_fake_bug("updateflags")
    bug.updateflags({"someflag": "someval"})

    # Some minor flag API tests
    assert "creation_date" in bug.get_flag_type("needinfo")
    assert bug.get_flag_type("NOPE") is None
    assert bug.get_flags("NOPE") is None
    assert bug.get_flag_status("NOPE") is None

    # Minor get_history_raw wrapper
    fakebz = tests.mockbackend.make_bz(rhbz=True,
        bug_history_args="data/mockargs/test_bug_api_history.txt",
        bug_history_return={},
        bug_get_args=None,
        bug_get_return="data/mockreturn/test_getbug_rhel.txt",
        bug_comments_args="data/mockargs/test_bug_api_comments.txt",
        bug_comments_return={"bugs": {"1165434": {"comments": []}}},
        bug_attachment_get_all_args=(
            "data/mockargs/test_bug_api_get_attachments.txt"),
        bug_attachment_get_all_return="data/mockreturn/test_attach_get2.txt",
    )

    # Stub API testing
    bug = fakebz.getbug(1165434)
    bug.get_history_raw()
    bug.getcomments()

    # Some hackery to hit a few attachment code paths
    bug.id = 663674
    attachments = bug.get_attachments()
    bug.attachments = attachments
    assert [469147, 470041, 502352] == bug.get_attachment_ids()



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

import tests
import tests.mockbackend
import tests.utils


###############################
# 'bugzilla new' mock testing #
###############################

def test_new(run_cli):
    # Test a simpler creation
    cmd = "bugzilla new --product FOOPROD --component FOOCOMP "
    cmd += "--summary 'Hey this is the title!' "
    cmd += "--comment 'This is the first comment!\nWith newline & stuff.' "
    cmd += "--keywords ADDKEY --groups FOOGROUP,BARGROUP "
    cmd += "--blocked 12345,6789 --cc foo@example.com --cc bar@example.com "
    cmd += "--dependson dependme --private "

    fakebz = tests.mockbackend.make_bz(
        bug_create_args="data/mockargs/test_new1.txt",
        bug_create_return={"id": 1694158},
        bug_get_args=None,
        bug_get_return="data/mockreturn/test_getbug.txt")
    out = run_cli(cmd, fakebz)
    tests.utils.diff_compare(out, "data/clioutput/test_new1.txt")

    # Test every option
    cmd = "bugzilla new --product FOOPROD --component FOOCOMP "
    cmd += "--summary 'Hey this is the title!' "
    cmd += "--comment 'This is the first comment!\nWith newline & stuff.' "
    cmd += "--keywords ADDKEY --groups FOOGROUP,BARGROUP "
    cmd += "--blocked 12345,6789 --cc foo@example.com --cc bar@example.com "
    cmd += "--dependson dependme --private "
    cmd += "--os linux --arch mips --severity high --priority low "
    cmd += "--url https://some.example.com "
    cmd += "--version 5.6.7 --alias somealias "
    cmd += "--sub-component FOOCOMP "
    cmd += "--assignee foo@example.com --qa_contact qa@example.com "
    cmd += "--comment-tag FOO "
    cmd += "--field foo=bar "

    fakebz = tests.mockbackend.make_bz(
        bug_create_args="data/mockargs/test_new2.txt",
        bug_create_return={"id": 1694158},
        bug_get_args=None,
        bug_get_return="data/mockreturn/test_getbug.txt",
        rhbz=True)
    out = run_cli(cmd, fakebz)
    tests.utils.diff_compare(out, "data/clioutput/test_new2.txt")



#
# Copyright Red Hat, Inc. 2012
#
# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.
#

"""
Unit tests that do permanent functional against a real bugzilla instances.
"""

import datetime
import inspect
import os
import random
import sys

import bugzilla

import tests
import tests.mockbackend
import tests.utils


RHURL = tests.CLICONFIG.REDHAT_URL or "bugzilla.stage.redhat.com"


def _split_int(s):
    return [int(i) for i in s.split(",")]


def _open_bz(**kwargs):
    return tests.utils.open_functional_bz(bugzilla.RHBugzilla, RHURL, kwargs)


if not _open_bz().logged_in:
    print("\nR/W tests require cached login credentials for url=%s\n" % RHURL)
    sys.exit(1)


def _check_have_admin(bz):
    funcname = inspect.stack()[1][3]

    # groupnames is empty for any user if our logged in user does not
    # have admin privs.
    # Check a known account that likely won't ever go away
    ret = bool(bz.getuser("anaconda-maint-list@redhat.com").groupnames)
    if not ret:
        print("\nNo admin privs, reduced testing of %s" % funcname)
    return ret


def test0LoggedInNoCreds():
    bz = _open_bz(use_creds=False)
    assert not bz.logged_in


def test0ClassDetection():
    bz = bugzilla.Bugzilla(RHURL, use_creds=False)
    assert bz.__class__ is bugzilla.RHBugzilla


def _makebug(run_cli, bz):
    component = "python-bugzilla"
    version = "rawhide"
    summary = ("python-bugzilla test basic bug %s" %
               datetime.datetime.today())
    newout = run_cli("bugzilla new "
        "--product Fedora --component %s --version %s "
        "--summary \"%s\" "
        "--comment \"Test bug from the python-bugzilla test suite\" "
        "--outputformat \"%%{bug_id}\"" %
        (component, version, summary), bz)

    assert len(newout.splitlines()) == 1
    bugid = int(newout.splitlines()[0])
    bug = bz.getbug(bugid)
    print("\nCreated bugid: %s" % bug.id)

    assert bug.component == component
    assert bug.version == version
    assert bug.summary == summary

    return bug


def test03NewBugBasic(run_cli, backends):
    """
    Create a bug with minimal amount of fields, then close it
    """
    bz = _open_bz(**backends)
    bug = _makebug(run_cli, bz)

    # Verify hasattr works
    assert hasattr(bug, "id")
    assert hasattr(bug, "bug_id")

    # Close the bug
    run_cli("bugzilla modify --close NOTABUG %s --minor-update" % bug.id, bz)
    bug.refresh()
    assert bug.status == "CLOSED"
    assert bug.resolution == "NOTABUG"


def test04NewBugAllFields(run_cli, backends):
    """
    Create a bug using all 'new' fields, check some values, close it
    """
    bz = _open_bz(**backends)

    summary = ("python-bugzilla test manyfields bug %s" %
               datetime.datetime.today())
    url = "http://example.com"
    osval = "Windows"
    cc = "triage@lists.fedoraproject.org"
    blocked = "461686,461687"
    dependson = "427301"
    comment = "Test bug from python-bugzilla test suite"
    sub_component = "Command-line tools (RHEL6)"
    alias = "pybz-%s" % datetime.datetime.today().strftime("%s")
    newout = run_cli("bugzilla new "
        "--product 'Red Hat Enterprise Linux 6' --version 6.0 "
        "--component lvm2 --sub-component '%s' "
        "--summary \"%s\" "
        "--comment \"%s\" "
        "--url %s --severity Urgent --priority Low --os %s "
        "--arch ppc --cc %s --blocked %s --dependson %s "
        "--alias %s "
        "--outputformat \"%%{bug_id}\"" %
        (sub_component, summary, comment, url,
         osval, cc, blocked, dependson, alias), bz)

    assert len(newout.splitlines()) == 1

    bugid = int(newout.splitlines()[0])
    bug = bz.getbug(bugid, extra_fields=["sub_components"])
    print("\nCreated bugid: %s" % bugid)

    assert bug.summary == summary
    assert bug.bug_file_loc == url
    assert bug.op_sys == osval
    assert bug.blocks == _split_int(blocked)
    assert bug.depends_on == _split_int(dependson)
    assert all([e in bug.cc for e in cc.split(",")])
    assert bug.longdescs[0]["text"] == comment
    assert bug.sub_components == {"lvm2": [sub_component]}
    assert bug.alias == [alias]

    # Close the bug

    # RHBZ makes it difficult to provide consistent semantics for
    # 'alias' update:
    # https://bugzilla.redhat.com/show_bug.cgi?id=1173114
    # alias += "-closed"
    run_cli("bugzilla modify "
        "--close WONTFIX %s " %
        bugid, bz)
    bug.refresh()
    assert bug.status == "CLOSED"
    assert bug.resolution == "WONTFIX"
    assert bug.alias == [alias]

    # Check bug's minimal history
    ret = bug.get_history_raw()
    assert len(ret["bugs"]) == 1
    assert len(ret["bugs"][0]["history"])


def test05ModifyStatus(run_cli, backends):
    """
    Modify status and comment fields for an existing bug
    """
    bz = _open_bz(**backends)
    bugid = "663674"
    cmd = "bugzilla modify %s " % bugid

    bug = bz.getbug(bugid)

    # We want to start with an open bug, so fix things
    if bug.status == "CLOSED":
        run_cli(cmd + "--status ASSIGNED", bz)
        bug.refresh()
        assert bug.status == "ASSIGNED"

    origstatus = bug.status

    # Set to ON_QA with a private comment
    status = "ON_QA"
    comment = ("changing status to %s at %s" %
               (status, datetime.datetime.today()))
    run_cli(cmd +
        "--status %s --comment \"%s\" --private" % (status, comment), bz)

    bug.refresh()
    assert bug.status == status
    assert bug.longdescs[-1]["is_private"] == 1
    assert bug.longdescs[-1]["text"] == comment

    # Close bug as DEFERRED with a private comment
    resolution = "DEFERRED"
    comment = ("changing status to CLOSED=%s at %s" %
               (resolution, datetime.datetime.today()))
    run_cli(cmd +
        "--close %s --comment \"%s\" --private" %
        (resolution, comment), bz)

    bug.refresh()
    assert bug.status == "CLOSED"
    assert bug.resolution == resolution
    assert bug.comments[-1]["is_private"] == 1
    assert bug.comments[-1]["text"] == comment

    # Close bug as dup with no comment
    dupeid = "461686"
    desclen = len(bug.longdescs)
    run_cli(cmd +
        "--close DUPLICATE --dupeid %s" % dupeid, bz)

    bug.refresh()
    assert bug.dupe_of == int(dupeid)
    assert len(bug.longdescs) == (desclen + 1)
    assert "marked as a duplicate" in bug.longdescs[-1]["text"]

    # bz.setstatus test
    comment = ("adding lone comment at %s" % datetime.datetime.today())
    bug.setstatus("POST", comment=comment, private=True)
    bug.refresh()
    assert bug.longdescs[-1]["is_private"] == 1
    assert bug.longdescs[-1]["text"] == comment
    assert bug.status == "POST"

    # bz.close test
    fixed_in = str(datetime.datetime.today())
    bug.close("ERRATA", fixedin=fixed_in)
    bug.refresh()
    assert bug.status == "CLOSED"
    assert bug.resolution == "ERRATA"
    assert bug.fixed_in == fixed_in

    # bz.addcomment test
    comment = ("yet another test comment %s" % datetime.datetime.today())
    bug.addcomment(comment, private=False)
    bug.refresh()
    assert bug.longdescs[-1]["text"] == comment
    assert bug.longdescs[-1]["is_private"] == 0

    # Confirm comments is same as getcomments
    assert bug.comments == bug.getcomments()

    # Reset state
    run_cli(cmd + "--status %s" % origstatus, bz)
    bug.refresh()
    assert bug.status == origstatus


def test06ModifyEmails(run_cli, backends):
    """
    Modify cc, assignee, qa_contact for existing bug
    """
    bz = _open_bz(**backends)
    bugid = "663674"
    cmd = "bugzilla modify %s " % bugid

    bug = bz.getbug(bugid)

    origcc = bug.cc

    # Test CC list and reset it
    email1 = "triage@lists.fedoraproject.org"
    email2 = "crobinso@redhat.com"
    bug.deletecc(origcc)
    run_cli(cmd + "--cc %s --cc %s" % (email1, email2), bz)
    bug.addcc(email1)

    bug.refresh()
    assert email1 in bug.cc
    assert email2 in bug.cc
    assert len(bug.cc) == 2

    run_cli(cmd + "--cc=-%s" % email1, bz)
    bug.refresh()
    assert email1 not in bug.cc

    # Test assigned target
    run_cli(cmd + "--assignee %s" % email1, bz)
    bug.refresh()
    assert bug.assigned_to == email1

    # Test QA target
    run_cli(cmd + "--qa_contact %s" % email1, bz)
    bug.refresh()
    assert bug.qa_contact == email1

    # Reset values
    bug.deletecc(bug.cc)
    run_cli(cmd + "--reset-qa-contact --reset-assignee", bz)

    bug.refresh()
    assert bug.cc == []
    assert bug.assigned_to == "crobinso@redhat.com"
    assert bug.qa_contact == "extras-qa@fedoraproject.org"


def test07ModifyMultiFlags(run_cli, backends):
    """
    Modify flags and fixed_in for 2 bugs
    """
    bz = _open_bz(**backends)
    bugid1 = "461686"
    bugid2 = "461687"
    cmd = "bugzilla modify %s %s " % (bugid1, bugid2)

    def flagstr(b):
        ret = []
        for flag in b.flags:
            ret.append(flag["name"] + flag["status"])
        return " ".join(sorted(ret))

    def cleardict_old(b):
        """
        Clear flag dictionary, for format meant for bug.updateflags
        """
        clearflags = {}
        for flag in b.flags:
            clearflags[flag["name"]] = "X"
        return clearflags

    def cleardict_new(b):
        """
        Clear flag dictionary, for format meant for update_bugs
        """
        clearflags = []
        for flag in b.flags:
            clearflags.append({"name": flag["name"], "status": "X"})
        return clearflags

    bug1 = bz.getbug(bugid1)
    if cleardict_old(bug1):
        bug1.updateflags(cleardict_old(bug1))
    bug2 = bz.getbug(bugid2)
    if cleardict_old(bug2):
        bug2.updateflags(cleardict_old(bug2))


    # Set flags and confirm
    setflags = "needinfo? requires_doc_text-"
    run_cli(cmd +
        " ".join([(" --flag " + f) for f in setflags.split()]), bz)

    bug1.refresh()
    bug2.refresh()

    assert flagstr(bug1) == setflags
    assert flagstr(bug2) == setflags
    assert bug1.get_flags("needinfo")[0]["status"] == "?"
    assert bug1.get_flag_status("requires_doc_text") == "-"

    # Clear flags
    if cleardict_new(bug1):
        bz.update_flags(bug1.id, cleardict_new(bug1))
    bug1.refresh()
    if cleardict_new(bug2):
        bz.update_flags(bug2.id, cleardict_new(bug2))
    bug2.refresh()

    assert cleardict_old(bug1) == {}
    assert cleardict_old(bug2) == {}

    # Set "Fixed In" field
    origfix1 = bug1.fixed_in
    origfix2 = bug2.fixed_in

    newfix = origfix1 and (origfix1 + "-new1") or "blippy1"
    if newfix == origfix2:
        newfix = origfix2 + "-2"

    run_cli(cmd + "--fixed_in=%s" % newfix, bz)

    bug1.refresh()
    bug2.refresh()
    assert bug1.fixed_in == newfix
    assert bug2.fixed_in == newfix

    # Reset fixed_in
    run_cli(cmd + "--fixed_in=\"-\"", bz)

    bug1.refresh()
    bug2.refresh()
    assert bug1.fixed_in == "-"
    assert bug2.fixed_in == "-"


def test07ModifyMisc(run_cli, backends):
    bugid = "461686"
    cmd = "bugzilla modify %s " % bugid
    bz = _open_bz(**backends)
    bug = bz.getbug(bugid)

    # modify --dependson
    run_cli(cmd + "--dependson 123456", bz)
    bug.refresh()
    assert 123456 in bug.depends_on
    run_cli(cmd + "--dependson =111222", bz)
    bug.refresh()
    assert [111222] == bug.depends_on
    run_cli(cmd + "--dependson=-111222", bz)
    bug.refresh()
    assert [] == bug.depends_on

    # modify --blocked
    run_cli(cmd + "--blocked 123,456", bz)
    bug.refresh()
    assert [123, 456] == bug.blocks
    run_cli(cmd + "--blocked =", bz)
    bug.refresh()
    assert [] == bug.blocks

    # modify --keywords
    run_cli(cmd + "--keywords +Documentation --keywords EasyFix", bz)
    bug.refresh()
    assert ["Documentation", "EasyFix"] == bug.keywords
    run_cli(cmd + "--keywords=-EasyFix --keywords=-Documentation",
                  bz)
    bug.refresh()
    assert [] == bug.keywords

    # modify --target_release
    # modify --target_milestone
    targetbugid = 492463
    targetbug = bz.getbug(targetbugid)
    targetcmd = "bugzilla modify %s " % targetbugid
    run_cli(targetcmd +
            "--target_milestone beta --target_release 6.2", bz)
    targetbug.refresh()
    assert targetbug.target_milestone == "beta"
    assert targetbug.target_release == ["6.2"]
    run_cli(targetcmd +
            "--target_milestone rc --target_release 6.10", bz)
    targetbug.refresh()
    assert targetbug.target_milestone == "rc"
    assert targetbug.target_release == ["6.10"]

    # modify --priority
    # modify --severity
    run_cli(cmd + "--priority low --severity high", bz)
    bug.refresh()
    assert bug.priority == "low"
    assert bug.severity == "high"
    run_cli(cmd + "--priority medium --severity medium", bz)
    bug.refresh()
    assert bug.priority == "medium"
    assert bug.severity == "medium"

    # modify --os
    # modify --platform
    # modify --version
    run_cli(cmd + "--version rawhide --os Windows --arch ppc "
            "--url http://example.com", bz)
    bug.refresh()
    assert bug.version == "rawhide"
    assert bug.op_sys == "Windows"
    assert bug.platform == "ppc"
    assert bug.url == "http://example.com"
    run_cli(cmd + "--version rawhide --os Linux --arch s390 "
            "--url http://example.com/fribby", bz)
    bug.refresh()
    assert bug.version == "rawhide"
    assert bug.op_sys == "Linux"
    assert bug.platform == "s390"
    assert bug.url == "http://example.com/fribby"

    # modify --field
    run_cli(cmd + "--field cf_fixed_in=foo-bar-1.2.3 \
                  --field=cf_release_notes=baz", bz)

    bug.refresh()
    assert bug.fixed_in == "foo-bar-1.2.3"
    assert bug.cf_release_notes == "baz"


def test08Attachments(run_cli, backends):
    tmpdir = "__test_attach_output"
    if tmpdir in os.listdir("."):
        os.system("rm -r %s" % tmpdir)
    os.mkdir(tmpdir)
    os.chdir(tmpdir)

    try:
        _test8Attachments(run_cli, backends)
    finally:
        os.chdir("..")
        os.system("rm -r %s" % tmpdir)


def _test8Attachments(run_cli, backends):
    """
    Get and set attachments for a bug
    """
    bz = _open_bz(**backends)
    cmd = "bugzilla attach "
    testfile = "../tests/data/bz-attach-get1.txt"

    # Add attachment as CLI option
    setbug = _makebug(run_cli, bz)
    setbug = bz.getbug(setbug.id, extra_fields=["attachments"])
    orignumattach = len(setbug.attachments)

    # Add attachment from CLI with mime guessing
    desc1 = "python-bugzilla cli upload %s" % datetime.datetime.today()
    out1 = run_cli(cmd + "%s --description \"%s\" --file %s" %
                         (setbug.id, desc1, testfile), bz)
    out1 = out1.splitlines()[-1]

    desc2 = "python-bugzilla cli upload %s" % datetime.datetime.today()
    out2 = run_cli(cmd + "%s --file test --summary \"%s\"" %
                         (setbug.id, desc2), bz, stdin=open(testfile).read())

    # Expected output format:
    #   Created attachment <attachid> on bug <bugid>

    setbug.refresh()
    assert len(setbug.attachments) == (orignumattach + 2)

    att1 = setbug.attachments[-2]
    attachid = att1["id"]
    assert att1["summary"] == desc1
    assert att1["id"] == int(out1.splitlines()[0].split()[2])
    assert att1["content_type"] == "text/plain"

    att2 = setbug.attachments[-1]
    assert att2["summary"] == desc2
    assert att2["id"] == int(out2.splitlines()[0].split()[2])
    assert att2["content_type"] == "application/octet-stream"

    # Set attachment flags
    assert att1["flags"] == []
    bz.updateattachmentflags(setbug.id, att2["id"], "review", status="+")
    setbug.refresh()

    assert len(setbug.attachments[-1]["flags"]) == 1
    assert setbug.attachments[-1]["flags"][0]["name"] == "review"
    assert setbug.attachments[-1]["flags"][0]["status"] == "+"

    bz.updateattachmentflags(setbug.id, setbug.attachments[-1]["id"],
                             "review", status="X")
    setbug.refresh()
    assert setbug.attachments[-1]["flags"] == []

    # Set attachment obsolete
    bz._backend.bug_attachment_update(  # pylint: disable=protected-access
        [setbug.attachments[-1]["id"]],
        {"is_obsolete": 1})
    setbug.refresh()
    assert setbug.attachments[-1]["is_obsolete"] == 1


    # Get attachment, verify content
    out = run_cli(cmd + "--get %s" % attachid, bz).splitlines()

    # Expect format:
    #   Wrote <filename>
    fname = out[0].split()[1].strip()

    assert len(out) == 1
    assert fname == "bz-attach-get1.txt"
    assert open(fname).read() == open(testfile).read()
    os.unlink(fname)

    # Get all attachments
    getbug = bz.getbug(setbug.id)
    getbug.autorefresh = True
    numattach = len(getbug.attachments)
    out = run_cli(cmd + "--getall %s" % getbug.id, bz).splitlines()

    assert len(out) == numattach
    fnames = [line.split(" ", 1)[1].strip() for line in out]
    assert len(fnames) == numattach
    for f in fnames:
        if not os.path.exists(f):
            raise AssertionError("filename '%s' not found" % f)
        os.unlink(f)

    # Get all attachments, but ignore obsolete
    ignorecmd = cmd + "--getall %s --ignore-obsolete" % getbug.id
    out = run_cli(ignorecmd, bz).splitlines()

    assert len(out) == (numattach - 1)
    fnames = [line.split(" ", 1)[1].strip() for line in out]
    assert len(fnames) == (numattach - 1)
    for f in fnames:
        if not os.path.exists(f):
            raise AssertionError("filename '%s' not found" % f)
        os.unlink(f)


def test09Whiteboards(run_cli, backends):
    bz = _open_bz(**backends)
    bug_id = "663674"
    cmd = "bugzilla modify %s " % bug_id
    bug = bz.getbug(bug_id)

    # Set all whiteboards
    initval = str(random.randint(1, 1024))
    run_cli(cmd +
            "--whiteboard =%sstatus "
            "--devel_whiteboard =%sdevel "
            "--internal_whiteboard '=%sinternal, security, foo security1' "
            "--qa_whiteboard =%sqa " %
            (initval, initval, initval, initval), bz)

    bug.refresh()
    assert bug.whiteboard == (initval + "status")
    assert bug.qa_whiteboard == (initval + "qa")
    assert bug.devel_whiteboard == (initval + "devel")
    assert (bug.internal_whiteboard ==
            (initval + "internal, security, foo security1"))

    # Modify whiteboards
    run_cli(cmd +
            "--whiteboard =foobar "
            "--qa_whiteboard _app "
            "--devel_whiteboard =pre-%s" % bug.devel_whiteboard, bz)

    bug.refresh()
    assert bug.qa_whiteboard == (initval + "qa" + " _app")
    assert bug.devel_whiteboard == ("pre-" + initval + "devel")
    assert bug.status_whiteboard == "foobar"

    # Verify that tag manipulation is smart about separator
    run_cli(cmd +
            "--qa_whiteboard=-_app "
            "--internal_whiteboard=-security,", bz)
    bug.refresh()

    assert bug.qa_whiteboard == (initval + "qa")
    assert bug.internal_whiteboard == (initval + "internal, foo security1")

    # Clear whiteboards
    update = bz.build_update(
        whiteboard="", devel_whiteboard="",
        internal_whiteboard="", qa_whiteboard="")
    bz.update_bugs(bug.id, update)

    bug.refresh()
    assert bug.whiteboard == ""
    assert bug.qa_whiteboard == ""
    assert bug.devel_whiteboard == ""
    assert bug.internal_whiteboard == ""


def test10Login(run_cli, monkeypatch):
    """
    Failed login test, gives us a bit more coverage
    """
    tests.utils.monkeypatch_getpass(monkeypatch)

    cmd = "bugzilla --no-cache-credentials --bugzilla %s" % RHURL
    # Implied login with --username and --password
    ret = run_cli("%s --user foobar@example.com "
            "--password foobar query -b 123456" % cmd,
            None, expectfail=True)
    assert "Login failed: " in ret

    # 'login' with explicit options
    ret = run_cli("%s --user foobar@example.com "
            "--password foobar login" % cmd,
            None, expectfail=True)
    assert "Login failed: " in ret

    # 'login' with positional options
    ret = run_cli("%s login foobar@example.com foobar" % cmd,
            None, expectfail=True)
    assert "Login failed: " in ret

    # bare 'login'
    stdinstr = "foobar@example.com\n\rfoobar\n\r"
    ret = run_cli("%s login" % cmd,
            None, expectfail=True, stdin=stdinstr)
    assert "Bugzilla Username:" in ret
    assert "Bugzilla Password:" in ret
    assert "Login failed: " in ret


def test11UserUpdate(backends):
    # This won't work if run by the same user we are using
    bz = _open_bz(**backends)
    email = "anaconda-maint-list@redhat.com"
    group = "fedora_contrib"

    have_admin = _check_have_admin(bz)

    user = bz.getuser(email)
    if have_admin:
        assert group in user.groupnames
    origgroups = user.groupnames

    # Test group_get
    try:
        group = bz.getgroup("fedora_contrib")
        group.refresh()
    except Exception as e:
        if have_admin:
            raise
        assert bugzilla.BugzillaError.get_bugzilla_error_code(e) == 805

    # Remove the group
    try:
        bz.updateperms(email, "remove", [group])
        user.refresh()
        assert group not in user.groupnames
    except Exception as e:
        if have_admin:
            raise
        assert "Sorry, you aren't a member" in str(e)

    # Re add it
    try:
        bz.updateperms(email, "add", group)
        user.refresh()
        assert group in user.groupnames
    except Exception as e:
        if have_admin:
            raise
        assert "Sorry, you aren't a member" in str(e)

    # Set groups
    try:
        newgroups = user.groupnames[:]
        if have_admin:
            newgroups.remove(group)
        bz.updateperms(email, "set", newgroups)
        user.refresh()
        assert group not in user.groupnames
    except Exception as e:
        if have_admin:
            raise
        assert "Sorry, you aren't a member" in str(e)

    # Reset everything
    try:
        bz.updateperms(email, "set", origgroups)
    except Exception as e:
        if have_admin:
            raise
        assert "Sorry, you aren't a member" in str(e)

    user.refresh()
    assert user.groupnames == origgroups

    # Try user create
    try:
        name = "pythonbugzilla-%s" % datetime.datetime.today()
        bz.createuser(name + "@example.com", name, name)
    except Exception as e:
        if have_admin:
            raise
        assert "Sorry, you aren't a member" in str(e)


def test11ComponentEditing(backends):
    bz = _open_bz(**backends)
    component = ("python-bugzilla-testcomponent-%s" %
                 str(random.randint(1, 1024 * 1024 * 1024)))
    basedata = {
        "product": "Fedora Documentation",
        "component": component,
    }

    have_admin = _check_have_admin(bz)

    def compare(data, newid):
        # pylint: disable=protected-access
        products = bz._proxy.Product.get({"names": [basedata["product"]]})
        compdata = None
        for c in products["products"][0]["components"]:
            if int(c["id"]) == int(newid):
                compdata = c
                break

        assert bool(compdata)
        assert data["component"] == compdata["name"]
        assert data["description"] == compdata["description"]
        assert data["initialowner"] == compdata["default_assigned_to"]
        assert data["initialqacontact"] == compdata["default_qa_contact"]
        assert data["is_active"] == compdata["is_active"]


    # Create component
    data = basedata.copy()
    data.update({
        "description": "foo test bar",
        "initialowner": "crobinso@redhat.com",
        "initialqacontact": "extras-qa@fedoraproject.org",
        "initialcclist": ["wwoods@redhat.com", "toshio@fedoraproject.org"],
        "is_active": True,
    })
    newid = None
    try:
        newid = bz.addcomponent(data)['id']
        print("Created product=%s component=%s" % (
            basedata["product"], basedata["component"]))
        compare(data, newid)
    except Exception as e:
        if have_admin:
            raise
        assert (("Sorry, you aren't a member" in str(e)) or
            # bugzilla 5 error string
            ("You are not allowed" in str(e)))

    # Edit component
    data = basedata.copy()
    data.update({
        "description": "hey new desc!",
        "initialowner": "extras-qa@fedoraproject.org",
        "initialqacontact": "virt-mgr-maint@redhat.com",
        "initialcclist": ["libvirt-maint@redhat.com",
                          "virt-maint@lists.fedoraproject.org"],
        "is_active": False,
    })
    try:
        bz.editcomponent(data)
        if newid is not None:
            compare(data, newid)
    except Exception as e:
        if bz.is_rest():
            # redhat REST does not support component editing
            assert "A REST API resource was not found" in str(e)
        elif have_admin:
            raise
        else:
            assert (("Sorry, you aren't a member" in str(e)) or
                # bugzilla 5 error string
                ("You are not allowed" in str(e)))


def test13SubComponents(backends):
    bz = _open_bz(**backends)
    # Long closed RHEL5 lvm2 bug. This component has sub_components
    bug = bz.getbug("185526")
    bug.autorefresh = True
    assert bug.component == "lvm2"

    bz.update_bugs(bug.id, bz.build_update(
        component="lvm2", sub_component="Command-line tools (RHEL5)"))
    bug.refresh()
    assert bug.sub_components == {"lvm2": ["Command-line tools (RHEL5)"]}

    bz.update_bugs(bug.id, bz.build_update(
        component="lvm2", sub_component="Default / Unclassified (RHEL5)"))
    bug.refresh()
    assert bug.sub_components == {"lvm2": [
        "Default / Unclassified (RHEL5)"]}


def _testExternalTrackers(bz):
    bugid = 461686
    ext_bug_id = 380489

    # Delete any existing external trackers to get to a known state
    ids = [bug['id'] for bug in bz.getbug(bugid).external_bugs]
    if ids != []:
        bz.remove_external_tracker(ids=ids)

    url = "https://bugzilla.mozilla.org"
    if bz.bz_ver_major < 5:
        url = "http://bugzilla.mozilla.org"

    # test adding tracker
    kwargs = {
        'ext_type_id': 6,
        'ext_type_url': url,
        'ext_type_description': 'Mozilla Foundation',
    }
    bz.add_external_tracker(bugid, ext_bug_id, **kwargs)
    added_bug = bz.getbug(bugid).external_bugs[0]
    assert added_bug['type']['id'] == kwargs['ext_type_id']
    assert added_bug['type']['url'] == kwargs['ext_type_url']
    assert (added_bug['type']['description'] ==
        kwargs['ext_type_description'])

    # test updating status, description, and priority by id
    kwargs = {
        'ids': bz.getbug(bugid).external_bugs[0]['id'],
        'ext_status': 'New Status',
        'ext_description': 'New Description',
        'ext_priority': 'New Priority'
    }
    bz.update_external_tracker(**kwargs)
    updated_bug = bz.getbug(bugid).external_bugs[0]
    assert updated_bug['ext_bz_bug_id'] == str(ext_bug_id)
    assert updated_bug['ext_status'] == kwargs['ext_status']
    assert updated_bug['ext_description'] == kwargs['ext_description']
    assert updated_bug['ext_priority'] == kwargs['ext_priority']

    # test removing tracker
    ids = [bug['id'] for bug in bz.getbug(bugid).external_bugs]
    assert len(ids) == 1
    bz.remove_external_tracker(ids=ids)
    ids = [bug['id'] for bug in bz.getbug(bugid).external_bugs]
    assert len(ids) == 0


def test14ExternalTrackersAddUpdateRemoveQuery(backends):
    bz = _open_bz(**backends)
    try:
        _testExternalTrackers(bz)
    except Exception as e:
        if not bz.is_rest():
            raise
        assert "No REST API available" in str(e)


def test15EnsureLoggedIn(run_cli, backends):
    bz = _open_bz(**backends)
    comm = "bugzilla --ensure-logged-in query --bug_id 979546"
    run_cli(comm, bz)

    # Test that we don't pollute the query dict with auth info
    query = {"id": [1234567]}
    origquery = query.copy()
    bz.query(query)
    assert query == origquery


def test16ModifyTags(run_cli, backends):
    bugid = "461686"
    cmd = "bugzilla modify %s " % bugid
    bz = _open_bz(**backends)
    bug = bz.getbug(bugid)

    try:
        if bug.tags:
            bz.update_tags(bug.id, tags_remove=bug.tags)
            bug.refresh()
            assert bug.tags == []

        run_cli(cmd + "--tags foo --tags +bar --tags baz", bz)
        bug.refresh()
        assert bug.tags == ["foo", "bar", "baz"]

        run_cli(cmd + "--tags=-bar", bz)
        bug.refresh()
        assert bug.tags == ["foo", "baz"]

        bz.update_tags(bug.id, tags_remove=bug.tags)
        bug.refresh()
        assert bug.tags == []
    except Exception as e:
        if not bz.is_rest():
            raise
        assert "No REST API available" in str(e)


def test17LoginAPIKey(backends):
    api_key = "somefakeapikey1234"
    bz = _open_bz(use_creds=False, api_key=api_key, **backends)
    try:
        assert bz.logged_in is False

        # Use this to trigger a warning about api_key
        bz.createbug(bz.build_createbug())
    except Exception as e:
        assert "The API key you specified is invalid" in str(e)



#!/usr/bin/env python
#
# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

# getbug_restapi.py:
#            Simple demonstration of connecting to bugzilla over the REST
#            API and printing some bug details.

import bugzilla

# public test instance of bugzilla.redhat.com. It's okay to make changes
URL = "bugzilla.stage.redhat.com"

# By default, if plain Bugzilla(URL) is invoked, the Bugzilla class will
# attempt to determine if XMLRPC or REST API is available, with a preference
# for XMLRPC for back compatability. But you can use the REST API
# with force_rest=True
bzapi = bugzilla.Bugzilla(URL, force_rest=True)

# After that, the bugzilla API can be used as normal. See getbug.py for
# some more info here.
bug = bzapi.getbug(427301)
print("Fetched bug #%s:" % bug.id)
print("  Product   = %s" % bug.product)
print("  Component = %s" % bug.component)
print("  Status    = %s" % bug.status)
print("  Resolution= %s" % bug.resolution)
print("  Summary   = %s" % bug.summary)



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

import json
import os
import re

import tests
import tests.mockbackend
import tests.utils


#################################
# 'bugzilla query' mock testing #
#################################

def test_query(run_cli):
    # bad field option
    fakebz = tests.mockbackend.make_bz()
    cmd = "bugzilla query --field FOO"
    out = run_cli(cmd, fakebz, expectfail=True)
    assert "Invalid field argument" in out

    # Simple query with some comma opts
    cmd = "bugzilla query "
    cmd += "--product foo --component foo,bar --bug_id 1234,2480"
    fakebz = tests.mockbackend.make_bz(
        bug_search_args="data/mockargs/test_query1.txt",
        bug_search_return="data/mockreturn/test_query1.txt")
    out = run_cli(cmd, fakebz)
    tests.utils.diff_compare(out, "data/clioutput/test_query1.txt")

    # RHBZ query with a ton of opts
    cmd = "bugzilla query "
    cmd += "--product foo --component foo,bar --bug_id 1234,2480 "
    cmd += "--keywords fribkeyword --fixed_in amifixed "
    cmd += "--qa_whiteboard some-example-whiteboard "
    cmd += "--cc foo@example.com --qa_contact qa@example.com "
    cmd += "--comment 'some comment string' "
    fakebz = tests.mockbackend.make_bz(rhbz=True,
        bug_search_args="data/mockargs/test_query1-rhbz.txt",
        bug_search_return="data/mockreturn/test_query1.txt")
    out = run_cli(cmd, fakebz)
    tests.utils.diff_compare(out, "data/clioutput/test_query1-rhbz.txt")

    # --emailtype handling
    cmd = "bugzilla query --cc foo@example.com --emailtype BAR "
    fakebz = tests.mockbackend.make_bz(rhbz=True,
        bug_search_args="data/mockargs/test_query2-rhbz.txt",
        bug_search_return="data/mockreturn/test_query1.txt")
    out = run_cli(cmd, fakebz)
    tests.utils.diff_compare(out, "data/clioutput/test_query2-rhbz.txt")

    # Same but with --ids output
    cmd = "bugzilla query --ids "
    cmd += "--product foo --component foo,bar --bug_id 1234,2480"
    fakebz = tests.mockbackend.make_bz(
        bug_search_args="data/mockargs/test_query1-ids.txt",
        bug_search_return="data/mockreturn/test_query1.txt")
    out = run_cli(cmd, fakebz)
    tests.utils.diff_compare(out, "data/clioutput/test_query1-ids.txt")

    # Same but with --raw output
    cmd = "bugzilla query --raw --bug_id 1165434"
    fakebz = tests.mockbackend.make_bz(
        bug_search_args="data/mockargs/test_query2.txt",
        bug_search_return={"bugs": [{"id": 1165434}]},
        bug_get_args=None,
        bug_get_return="data/mockreturn/test_getbug_rhel.txt")
    out = run_cli(cmd, fakebz)
    # Dictionary ordering is random, so scrub it from our output
    out = re.sub(r"\{.*\}", r"'DICT SCRUBBED'", out, re.MULTILINE)
    tests.utils.diff_compare(out, "data/clioutput/test_query2.txt")

    # Test a bunch of different combinations for code coverage
    cmd = "bugzilla query --status ALL --severity sev1,sev2 "
    cmd += "--outputformat='%{foo}:%{bar}::%{whiteboard}:"
    cmd += "%{flags}:%{flags_requestee}%{whiteboard:devel}::"
    cmd += "%{flag:needinfo}::%{comments}::%{external_bugs}'"
    fakebz = tests.mockbackend.make_bz(
        bug_search_args="data/mockargs/test_query3.txt",
        bug_search_return="data/mockreturn/test_getbug_rhel.txt")
    out = run_cli(cmd, fakebz)
    tests.utils.diff_compare(out, "data/clioutput/test_query3.txt")

    # Test --status DEV and --full
    cmd = "bugzilla query --status DEV --full"
    fakebz = tests.mockbackend.make_bz(
        bug_search_args="data/mockargs/test_query4.txt",
        bug_search_return="data/mockreturn/test_getbug_rhel.txt")
    out = run_cli(cmd, fakebz)
    tests.utils.diff_compare(out, "data/clioutput/test_query4.txt")

    # Test --status QE and --extra, and components-file
    compfile = os.path.dirname(__file__) + "/data/components_file.txt"
    cmd = "bugzilla query --status QE --extra "
    cmd += "--components_file %s" % compfile
    fakebz = tests.mockbackend.make_bz(
        bug_search_args="data/mockargs/test_query5.txt",
        bug_search_return="data/mockreturn/test_getbug_rhel.txt")
    out = run_cli(cmd, fakebz)
    tests.utils.diff_compare(out, "data/clioutput/test_query5.txt")

    # Test --status EOL and --oneline, and some --field usage
    cmd = "bugzilla query --status EOL --oneline "
    cmd += "--field FOO=1 --field=BAR=WIBBLE "
    fakebz = tests.mockbackend.make_bz(
        bug_search_args="data/mockargs/test_query6.txt",
        bug_search_return="data/mockreturn/test_getbug_rhel.txt",
        bug_get_args="data/mockargs/test_query_cve_getbug.txt",
        bug_get_return="data/mockreturn/test_query_cve_getbug.txt")
    out = run_cli(cmd, fakebz)
    tests.utils.diff_compare(out, "data/clioutput/test_query6.txt")

    # Test --status OPEN and --from-url
    url = "https://bugzilla.redhat.com/buglist.cgi?bug_status=NEW&bug_status=ASSIGNED&bug_status=MODIFIED&bug_status=ON_DEV&bug_status=ON_QA&bug_status=VERIFIED&bug_status=FAILS_QA&bug_status=RELEASE_PENDING&bug_status=POST&classification=Fedora&component=virt-manager&order=bug_status%2Cbug_id&product=Fedora&query_format=advanced"  # noqa
    cmd = "bugzilla query --status OPEN --from-url %s" % url
    fakebz = tests.mockbackend.make_bz(
        bug_search_args="data/mockargs/test_query7.txt",
        bug_search_return="data/mockreturn/test_getbug_rhel.txt")
    out = run_cli(cmd, fakebz)
    tests.utils.diff_compare(out, "data/clioutput/test_query7.txt")

    # Test --json output
    cmd = "bugzilla query --json --id 1165434"
    fakebz = tests.mockbackend.make_bz(
        bug_search_args="data/mockargs/test_query8.txt",
        bug_search_return={"bugs": [{"id": 1165434}]},
        bug_get_args=None,
        bug_get_return="data/mockreturn/test_getbug_rhel.txt")
    out = run_cli(cmd, fakebz)
    tests.utils.diff_compare(tests.utils.sanitize_json(out),
        "data/clioutput/test_query8.txt")
    assert json.loads(out)

    # Test --json output
    cmd = ("bugzilla query --json --id 1165434 "
           "--includefield foo --includefield bar "
           "--excludefield excludeme "
           "--extrafield extrame1 --extrafield extrame2 ")
    fakebz = tests.mockbackend.make_bz(rhbz=True,
        bug_search_args="data/mockargs/test_query9.txt",
        bug_search_return={"bugs": [{"id": 1165434}]},
        bug_get_args="data/mockargs/test_getbug_query9.txt",
        bug_get_return="data/mockreturn/test_getbug_rhel.txt")
    out = run_cli(cmd, fakebz)
    tests.utils.diff_compare(tests.utils.sanitize_json(out),
        "data/clioutput/test_query9.txt")
    assert json.loads(out)


    # Test every remaining option
    cmd = "bugzilla query "
    cmd += "--sub-component FOOCOMP "
    cmd += "--version 5.6.7 --reporter me@example.com "
    cmd += "--summary 'search summary' "
    cmd += "--assignee bar@example.com "
    cmd += "--blocked 12345 --dependson 23456 "
    cmd += "--keywords FOO --keywords_type substring "
    cmd += "--url https://example.com --url_type sometype "
    cmd += "--target_release foo --target_milestone bar "
    cmd += "--quicksearch 1 --savedsearch 2 --savedsearch-sharer-id 3 "
    cmd += "--tags +foo --flag needinfo --alias somealias "
    cmd += "--devel_whiteboard DEVBOARD "
    cmd += "--priority wibble "
    cmd += "--fixed_in 5.5.5 --fixed_in_type substring "
    cmd += "--whiteboard FOO --status_whiteboard_type substring "
    fakebz = tests.mockbackend.make_bz(
        bug_search_args="data/mockargs/test_query10.txt",
        bug_search_return="data/mockreturn/test_getbug_rhel.txt",
        rhbz=True)
    out = run_cli(cmd, fakebz)
    tests.utils.diff_compare(out, "data/clioutput/test_query10.txt")



#
# Copyright Red Hat, Inc. 2012
#
# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.
#

"""
Test miscellaneous API bits
"""

import tests
import tests.mockbackend


def test_externalbugs():
    # Basic API testing of the ExternalBugs wrappers
    fakebz = tests.mockbackend.make_bz(
        externalbugs_add_args="data/mockargs/test_externalbugs_add.txt",
        externalbugs_add_return={},
        externalbugs_update_args="data/mockargs/test_externalbugs_update.txt",
        externalbugs_update_return={},
        externalbugs_remove_args="data/mockargs/test_externalbugs_remove.txt",
        externalbugs_remove_return={})

    fakebz.add_external_tracker(
        bug_ids=[1234, 5678],
        ext_bz_bug_id="externalid",
        ext_type_id="launchpad",
        ext_type_description="some-bug-add-description",
        ext_type_url="https://example.com/launchpad/1234",
        ext_status="CLOSED",
        ext_description="link to launchpad",
        ext_priority="bigly")

    fakebz.update_external_tracker(
        ids=["external1", "external2"],
        ext_bz_bug_id="externalid-update",
        ext_type_id="mozilla",
        ext_type_description="some-bug-update",
        ext_type_url="https://mozilla.foo/bar/5678",
        ext_status="OPEN",
        bug_ids=["some", "bug", "id"],
        ext_description="link to mozilla",
        ext_priority="like, really bigly")

    fakebz.remove_external_tracker(
        ids="remove1",
        ext_bz_bug_id="99999",
        ext_type_id="footype",
        ext_type_description="foo-desc",
        ext_type_url="foo-url",
        bug_ids="blah")



import unittest
from utils import *
from ksc import Ksc
from mock import Mock, patch


class ReadListTest(unittest.TestCase):
    """
    Test reading stablelist
    """
    def runTest(self):
        data, _ = read_list("x86_64", "kabi-current")
        assert len(data) != 0


class ReadTotalListTest(unittest.TestCase):
    """
    Test reading all symbol names
    """
    def runTest(self):
        data = read_total_list()
        assert len(data) != 0


class RunCommandTest(unittest.TestCase):
    """
    To test our own set function
    """
    def runTest(self):
        data = run('uname -a')
        self.assertTrue(data.startswith('Linux'))


class GetConfigTest(unittest.TestCase):
    """
    To test our own set function
    """
    def runTest(self):
        data = getconfig('./data/ksc.conf', True)
        assert 'user' in data
        assert 'partner' in data
        assert 'group' in data
        assert 'server' in data


class CreateBugTest(unittest.TestCase):
    """
    Code to test createbug function
    """
    def runTest(self):
        bugid = createbug('./data/ksc.conf', 'x86_64', True)  # This is mock


class ParseKOTest(unittest.TestCase):
    """
    Code to test parse_ko
    """
    @patch('ksc.run')
    def runTest(self, mock_run):
        mock_run.return_value = 'U add_disk\nU add_drv\nU call_rcu_bh'
        k = Ksc(mock=True)
        k.read_data('x86_64', 'kabi-current')
        k.parse_ko('./ksc.py')
        assert len(k.all_symbols_used) == 2
        assert len(k.nonstable_symbols_used) == 1
        assert len(k.stable_symbols) == 1


if __name__ == '__main__':
    unittest.main()



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

import inspect

import bugzilla
from bugzilla._backendbase import _BackendBase

import tests.utils


# pylint: disable=abstract-method,arguments-differ


class BackendMock(_BackendBase):
    _version = None

    def bugzilla_version(self):
        return {"version": self._version}

    def __helper(self, args):
        # Grab the calling function name and use it to generate
        # input and output variable names for the class. So if this
        # is called from bug_get, we look for:
        #   self._bug_get_args
        #   self._bug_get_return
        prevfuncname = inspect.stack()[1][3]
        func_args = getattr(self, "_%s_args" % prevfuncname)
        func_return = getattr(self, "_%s_return" % prevfuncname)
        if isinstance(func_return, BaseException):
            raise func_return

        filename = None
        expect_out = func_args
        if isinstance(func_args, str):
            filename = func_args
            expect_out = None

        # Hack to strip out attachment content from the generated
        # test output, because it doesn't play well with the test
        # suite running on python2
        if "content-disposition" in str(args):
            largs = list(args)
            largs[1] = "STRIPPED-BY-TESTSUITE"
            args = tuple(largs)

        if filename or expect_out:
            tests.utils.diff_compare(args, filename, expect_out)

        if isinstance(func_return, dict):
            return func_return

        returnstr = open(tests.utils.tests_path(func_return)).read()
        return eval(returnstr)  # pylint: disable=eval-used

    def bug_attachment_create(self, *args):
        return self.__helper(args)
    def bug_attachment_get(self, *args):
        return self.__helper(args)
    def bug_attachment_get_all(self, *args):
        return self.__helper(args)
    def bug_attachment_update(self, *args):
        return self.__helper(args)

    def bug_comments(self, *args):
        return self.__helper(args)
    def bug_create(self, *args):
        return self.__helper(args)
    def bug_history(self, *args):
        return self.__helper(args)
    def bug_get(self, *args):
        return self.__helper(args)
    def bug_fields(self, *args):
        return self.__helper(args)
    def bug_search(self, *args):
        return self.__helper(args)
    def bug_update(self, *args):
        return self.__helper(args)
    def bug_update_tags(self, *args):
        return self.__helper(args)

    def component_create(self, *args):
        return self.__helper(args)
    def component_get(self, *args):
        return self.__helper(args)
    def component_update(self, *args):
        return self.__helper(args)

    def group_get(self, *args):
        return self.__helper(args)

    def externalbugs_add(self, *args):
        return self.__helper(args)
    def externalbugs_update(self, *args):
        return self.__helper(args)
    def externalbugs_remove(self, *args):
        return self.__helper(args)

    def product_get(self, *args):
        return self.__helper(args)
    def product_get_accessible(self, *args):
        return self.__helper(args)
    def product_get_enterable(self, *args):
        return self.__helper(args)
    def product_get_selectable(self, *args):
        return self.__helper(args)

    def user_create(self, *args):
        return self.__helper(args)
    def user_get(self, *args):
        return self.__helper(args)
    def user_login(self, *args):
        return self.__helper(args)
    def user_logout(self, *args):
        return self.__helper(args)
    def user_update(self, *args):
        return self.__helper(args)


def _make_backend_class(version="6.0.0", **kwargs):

    class TmpBackendClass(BackendMock):
        _version = version

    for key, val in kwargs.items():
        setattr(TmpBackendClass, "_%s" % key, val)

    return TmpBackendClass


def make_bz(bz_kwargs=None, rhbz=False, **kwargs):
    bz_kwargs = (bz_kwargs or {}).copy()
    if "url" in bz_kwargs:
        raise RuntimeError("Can't set 'url' in mock make_bz, use connect()")

    if "use_creds" not in bz_kwargs:
        bz_kwargs["use_creds"] = False

    bz = bugzilla.Bugzilla(url=None, **bz_kwargs)
    backendclass = _make_backend_class(**kwargs)

    def _get_backend_class(url):
        return backendclass, bugzilla.Bugzilla.fix_url(url)

    # pylint: disable=protected-access
    bz._get_backend_class = _get_backend_class

    url = "https:///TESTSUITEMOCK"
    if rhbz:
        url += "?fakeredhat=bugzilla.redhat.com"
    bz.connect(url)
    return bz



#
# Copyright Red Hat, Inc. 2012
#
# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.
#

"""
Test miscellaneous API bits
"""

import os
import shutil
import tempfile

import tests
import tests.mockbackend
import tests.utils


def test_tokenfile(monkeypatch):
    dirname = os.path.dirname(__file__)
    monkeypatch.setitem(os.environ, "HOME", dirname + "/data/homedir")

    bz = tests.mockbackend.make_bz(bz_kwargs={"use_creds": True})
    token = dirname + "/data/homedir/.cache/python-bugzilla/bugzillatoken"

    assert token == bz.tokenfile
    del(bz.tokenfile)
    assert bz.tokenfile is None
    assert bz.cookiefile is None


def test_readconfig():
    # Testing for bugzillarc handling
    bzapi = tests.mockbackend.make_bz(version="4.4.0", rhbz=True)
    bzapi.url = "example.com"
    temp = tempfile.NamedTemporaryFile(mode="w")

    def _check(user, password, api_key, cert):
        assert bzapi.user == user
        assert bzapi.password == password
        assert bzapi.api_key == api_key
        assert bzapi.cert == cert

    def _write(c):
        temp.seek(0)
        temp.write(c)
        temp.flush()
        return temp.name

    # Check readconfig normal usage
    content = """
[example.com]
foo=1
user=test1
password=test2
api_key=123abc
cert=/a/b/c
someunknownkey=someval
"""
    bzapi.readconfig(_write(content))
    _check("test1", "test2", "123abc", "/a/b/c")

    # Check loading a different URL, that values aren't overwritten
    content = """
[foo.example.com]
user=test3
password=test4
api_key=567abc
cert=/newpath
"""
    bzapi.readconfig(_write(content))
    _check("test1", "test2", "123abc", "/a/b/c")

    # Change URL, but check readconfig with overwrite=False
    bzapi.url = "foo.example.com"
    bzapi.readconfig(temp.name, overwrite=False)
    _check("test1", "test2", "123abc", "/a/b/c")

    # With default overwrite=True, values will be updated
    # Alter the config to have a / in the hostname, which hits different code
    content = content.replace("example.com", "example.com/xmlrpc.cgi")
    bzapi.url = "https://foo.example.com/xmlrpc.cgi"
    bzapi.readconfig(_write(content))
    _check("test3", "test4", "567abc", "/newpath")

    # Confirm nothing overwritten for a totally different URL
    bzapi.user = None
    bzapi.password = None
    bzapi.api_key = None
    bzapi.cert = None
    bzapi.url = "bugzilla.redhat.com"
    bzapi.readconfig(temp.name)
    _check(None, None, None, None)

    # Test confipath overwrite
    assert [temp.name] == bzapi.configpath
    del(bzapi.configpath)
    assert [] == bzapi.configpath
    bzapi.readconfig()
    _check(None, None, None, None)


def test_authfiles_saving(monkeypatch):
    tmpdir = tempfile.mkdtemp()
    try:
        monkeypatch.setitem(os.environ, "HOME", tmpdir)

        bzapi = tests.mockbackend.make_bz(
            bz_kwargs={"use_creds": True, "cert": "foo-fake-cert"})
        bzapi.connect("https://example.com/fakebz")

        bzapi.cert = "foo-fake-path"
        backend = bzapi._backend  # pylint: disable=protected-access
        bsession = backend._bugzillasession  # pylint: disable=protected-access
        btokencache = bzapi._tokencache   # pylint: disable=protected-access

        # token testing, with repetitions to hit various code paths
        btokencache.set_value(bzapi.url, None)
        assert "Bugzilla_token" not in bsession.get_auth_params()
        btokencache.set_value(bzapi.url, "MY-FAKE-TOKEN")
        assert bsession.get_auth_params()["Bugzilla_token"] == "MY-FAKE-TOKEN"
        btokencache.set_value(bzapi.url, "MY-FAKE-TOKEN")
        btokencache.set_value(bzapi.url, None)
        assert "Bugzilla_token" not in bsession.get_auth_params()
        btokencache.set_value(bzapi.url, "MY-FAKE-TOKEN")

        dirname = os.path.dirname(__file__) + "/data/authfiles/"
        output_token = dirname + "output-token.txt"
        tests.utils.diff_compare(open(bzapi.tokenfile).read(), output_token)

        # Make sure file can re-read them and not error
        bzapi = tests.mockbackend.make_bz(
            bz_kwargs={"use_creds": True,
                       "tokenfile": output_token})
        assert bzapi.tokenfile == output_token

        # Test rcfile writing for api_key
        rcfile = bzapi._rcfile  # pylint: disable=protected-access
        bzapi.url = "https://example.com/fake"
        rcfile.save_api_key(bzapi.url, "TEST-API-KEY")
        rcfilepath = tmpdir + "/.config/python-bugzilla/bugzillarc"
        assert rcfile.get_configpaths()[-1] == rcfilepath
        tests.utils.diff_compare(open(rcfilepath).read(),
            dirname + "output-bugzillarc.txt")

        # Use that generated rcfile to test default URL lookup
        fakeurl = "http://foo.bar.baz/wibble"
        open(rcfilepath, "w").write("\n[DEFAULT]\nurl = %s" % fakeurl)
        assert bzapi.get_rcfile_default_url() == fakeurl
    finally:
        shutil.rmtree(tmpdir)


def test_authfiles_nowrite():
    # Setting values tokenfile is None, should be fine
    bzapi = tests.mockbackend.make_bz(bz_kwargs={"use_creds": False})
    bzapi.connect("https://example.com/foo")
    btokencache = bzapi._tokencache   # pylint: disable=protected-access
    rcfile = bzapi._rcfile  # pylint: disable=protected-access

    btokencache.set_value(bzapi.url, "NEW-TOKEN-VALUE")
    assert rcfile.save_api_key(bzapi.url, "fookey") is None



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

from logging import getLogger
import sys
from xmlrpc.client import (Binary, Fault, ProtocolError,
                           ServerProxy, Transport)

from requests import RequestException

from ._backendbase import _BackendBase
from .exceptions import BugzillaError
from ._util import listify


log = getLogger(__name__)


class _BugzillaXMLRPCTransport(Transport):
    def __init__(self, bugzillasession):
        if hasattr(Transport, "__init__"):
            Transport.__init__(self, use_datetime=False)

        self.__bugzillasession = bugzillasession
        self.__bugzillasession.set_xmlrpc_defaults()
        self.__seen_valid_xml = False

        # Override Transport.user_agent
        self.user_agent = self.__bugzillasession.get_user_agent()


    ############################
    # Bugzilla private helpers #
    ############################

    def __request_helper(self, url, request_body):
        """
        A helper method to assist in making a request and parsing the response.
        """
        response = None
        # pylint: disable=try-except-raise
        # pylint: disable=raise-missing-from
        try:
            response = self.__bugzillasession.request(
                "POST", url, data=request_body)

            return self.parse_response(response)
        except RequestException as e:
            if not response:
                raise
            raise ProtocolError(  # pragma: no cover
                url, response.status_code, str(e), response.headers)
        except Fault:
            raise
        except Exception:
            msg = str(sys.exc_info()[1])
            if not self.__seen_valid_xml:
                msg += "\nThe URL may not be an XMLRPC URL: %s" % url
            e = BugzillaError(msg)
            # pylint: disable=attribute-defined-outside-init
            e.__traceback__ = sys.exc_info()[2]
            # pylint: enable=attribute-defined-outside-init
            raise e


    ######################
    # Tranport overrides #
    ######################

    def parse_response(self, response):
        """
        Override Transport.parse_response
        """
        parser, unmarshaller = self.getparser()
        msg = response.text.encode('utf-8')
        try:
            parser.feed(msg)
        except Exception:  # pragma: no cover
            log.debug("Failed to parse this XMLRPC response:\n%s", msg)
            raise

        self.__seen_valid_xml = True
        parser.close()
        return unmarshaller.close()

    def request(self, host, handler, request_body, verbose=0):
        """
        Override Transport.request
        """
        # Setting self.verbose here matches overrided request() behavior
        # pylint: disable=attribute-defined-outside-init
        self.verbose = verbose

        url = "%s://%s%s" % (self.__bugzillasession.get_scheme(),
                host, handler)

        # xmlrpclib fails to escape \r
        request_body = request_body.replace(b'\r', b'&#xd;')

        return self.__request_helper(url, request_body)


class _BugzillaXMLRPCProxy(ServerProxy, object):
    """
    Override of xmlrpc ServerProxy, to insert bugzilla API auth
    into the XMLRPC request data
    """
    def __init__(self, uri, bugzillasession, *args, **kwargs):
        self.__bugzillasession = bugzillasession
        transport = _BugzillaXMLRPCTransport(self.__bugzillasession)
        ServerProxy.__init__(self, uri, transport, *args, **kwargs)

    def _ServerProxy__request(self, methodname, params):
        """
        Overrides ServerProxy _request method
        """
        # params is a singleton tuple, enforced by xmlrpc.client.dumps
        newparams = params and params[0].copy() or {}

        log.debug("XMLRPC call: %s(%s)", methodname, newparams)
        authparams = self.__bugzillasession.get_auth_params()
        authparams.update(newparams)

        # pylint: disable=no-member
        ret = ServerProxy._ServerProxy__request(
            self, methodname, (authparams,))
        # pylint: enable=no-member

        return ret


class _BackendXMLRPC(_BackendBase):
    """
    Internal interface for direct calls to bugzilla's XMLRPC API
    """
    def __init__(self, url, bugzillasession):
        _BackendBase.__init__(self, url, bugzillasession)
        self._xmlrpc_proxy = _BugzillaXMLRPCProxy(url, self._bugzillasession)

    def get_xmlrpc_proxy(self):
        return self._xmlrpc_proxy
    def is_xmlrpc(self):
        return True

    def bugzilla_version(self):
        return self._xmlrpc_proxy.Bugzilla.version()

    def bug_attachment_get(self, attachment_ids, paramdict):
        data = paramdict.copy()
        data["attachment_ids"] = listify(attachment_ids)
        return self._xmlrpc_proxy.Bug.attachments(data)
    def bug_attachment_get_all(self, bug_ids, paramdict):
        data = paramdict.copy()
        data["ids"] = listify(bug_ids)
        return self._xmlrpc_proxy.Bug.attachments(data)
    def bug_attachment_create(self, bug_ids, data, paramdict):
        pdata = paramdict.copy()
        pdata["ids"] = listify(bug_ids)
        if data is not None and "data" not in paramdict:
            pdata["data"] = Binary(data)
        return self._xmlrpc_proxy.Bug.add_attachment(pdata)
    def bug_attachment_update(self, attachment_ids, paramdict):
        data = paramdict.copy()
        data["ids"] = listify(attachment_ids)
        return self._xmlrpc_proxy.Bug.update_attachment(data)

    def bug_comments(self, bug_ids, paramdict):
        data = paramdict.copy()
        data["ids"] = listify(bug_ids)
        return self._xmlrpc_proxy.Bug.comments(data)
    def bug_create(self, paramdict):
        return self._xmlrpc_proxy.Bug.create(paramdict)
    def bug_fields(self, paramdict):
        return self._xmlrpc_proxy.Bug.fields(paramdict)
    def bug_get(self, bug_ids, aliases, paramdict):
        data = paramdict.copy()
        data["ids"] = listify(bug_ids) or []
        data["ids"] += listify(aliases) or []
        return self._xmlrpc_proxy.Bug.get(data)
    def bug_history(self, bug_ids, paramdict):
        data = paramdict.copy()
        data["ids"] = listify(bug_ids)
        return self._xmlrpc_proxy.Bug.history(data)
    def bug_search(self, paramdict):
        return self._xmlrpc_proxy.Bug.search(paramdict)
    def bug_update(self, bug_ids, paramdict):
        data = paramdict.copy()
        data["ids"] = listify(bug_ids)
        return self._xmlrpc_proxy.Bug.update(data)
    def bug_update_tags(self, bug_ids, paramdict):
        data = paramdict.copy()
        data["ids"] = listify(bug_ids)
        return self._xmlrpc_proxy.Bug.update_tags(data)

    def component_create(self, paramdict):
        return self._xmlrpc_proxy.Component.create(paramdict)
    def component_update(self, paramdict):
        return self._xmlrpc_proxy.Component.update(paramdict)

    def externalbugs_add(self, paramdict):
        return self._xmlrpc_proxy.ExternalBugs.add_external_bug(paramdict)
    def externalbugs_update(self, paramdict):
        return self._xmlrpc_proxy.ExternalBugs.update_external_bug(paramdict)
    def externalbugs_remove(self, paramdict):
        return self._xmlrpc_proxy.ExternalBugs.remove_external_bug(paramdict)

    def group_get(self, paramdict):
        return self._xmlrpc_proxy.Group.get(paramdict)

    def product_get(self, paramdict):
        return self._xmlrpc_proxy.Product.get(paramdict)
    def product_get_accessible(self):
        return self._xmlrpc_proxy.Product.get_accessible_products()
    def product_get_enterable(self):
        return self._xmlrpc_proxy.Product.get_enterable_products()
    def product_get_selectable(self):
        return self._xmlrpc_proxy.Product.get_selectable_products()

    def user_create(self, paramdict):
        return self._xmlrpc_proxy.User.create(paramdict)
    def user_get(self, paramdict):
        return self._xmlrpc_proxy.User.get(paramdict)
    def user_login(self, paramdict):
        return self._xmlrpc_proxy.User.login(paramdict)
    def user_logout(self):
        return self._xmlrpc_proxy.User.logout()
    def user_update(self, paramdict):
        return self._xmlrpc_proxy.User.update(paramdict)



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

import locale
import logging
import os

import pytest

import tests
import tests.utils

import bugzilla


# pytest plugin adding custom options. Hooks are documented here:
# https://docs.pytest.org/en/latest/writing_plugins.html

def pytest_addoption(parser):
    parser.addoption("--ro-functional", action="store_true", default=False,
            help=("Run readonly functional tests against actual "
                  "bugzilla instances. This will be very slow."))
    parser.addoption("--rw-functional", action="store_true", default=False,
            help=("Run read/write functional tests against actual bugzilla "
                  "instances. As of now this only runs against "
                  "bugzilla.stage.redhat.com, which requires an RH "
                  "bugzilla account with cached login creds. This will "
                  "also be very slow."))
    parser.addoption("--redhat-url",
            help="Redhat bugzilla URL to use for ro/rw_functional tests")
    parser.addoption("--pybz-debug", action="store_true", default=False,
            help=("Enable python-bugzilla debug output. This may break "
                  "output comparison tests."))
    parser.addoption("--regenerate-output",
            action="store_true", default=False,
            help=("Force regeneration of generated test output"))
    parser.addoption("--only-rest", action="store_true", default=False)
    parser.addoption("--only-xmlrpc", action="store_true", default=False)


def pytest_ignore_collect(path, config):
    has_ro = config.getoption("--ro-functional")
    has_rw = config.getoption("--rw-functional")

    base = os.path.basename(str(path))
    is_ro = base == "test_ro_functional.py"
    is_rw = base == "test_rw_functional.py"
    if is_ro and not has_ro:
        return True
    if is_rw and not has_rw:
        return True


def pytest_configure(config):
    try:
        # Needed for test reproducibility on systems not using a UTF-8 locale
        locale.setlocale(locale.LC_ALL, 'C')
        locale.setlocale(locale.LC_CTYPE, 'en_US.UTF-8')
    except Exception as e:
        print("Error setting locale: %s" % str(e))

    if config.getoption("--redhat-url"):
        tests.CLICONFIG.REDHAT_URL = config.getoption("--redhat-url")
    if config.getoption("--pybz-debug"):
        logging.getLogger(bugzilla.__name__).setLevel(logging.DEBUG)
        os.environ["__BUGZILLA_UNITTEST_DEBUG"] = "1"
    if config.getoption("--regenerate-output"):
        tests.CLICONFIG.REGENERATE_OUTPUT = config.getoption(
            "--regenerate-output")
    if config.getoption("--only-rest"):
        tests.CLICONFIG.ONLY_REST = True
    if config.getoption("--only-xmlrpc"):
        tests.CLICONFIG.ONLY_XMLRPC = True

    if (config.getoption("--ro-functional") or
        config.getoption("--rw-functional")):
        config.option.verbose = 2


def pytest_generate_tests(metafunc):
    """
    If a test requests the 'backends' fixture, run that test with both
    force_rest=True and force_xmlrpc=True Bugzilla options
    """
    if 'backends' in metafunc.fixturenames:
        values = []
        ids = []
        if not tests.CLICONFIG.ONLY_REST:
            values.append({"force_xmlrpc": True})
            ids.append("XMLRPC")
        if not tests.CLICONFIG.ONLY_XMLRPC:
            values.append({"force_rest": True})
            ids.append("REST")
        metafunc.parametrize("backends", values, ids=ids, scope="session")


@pytest.fixture
def run_cli(capsys, monkeypatch):
    """
    Custom pytest fixture to pass a function for testing
    a bugzilla cli command.
    """
    def _do_run(*args, **kwargs):
        return tests.utils.do_run_cli(capsys, monkeypatch, *args, **kwargs)
    return _do_run



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

import tempfile

import pytest

import bugzilla

import tests
import tests.mockbackend
import tests.utils


#################################
# 'bugzilla login' mock testing #
#################################

def test_login(run_cli):
    cmd = "bugzilla login FOO BAR"

    fakebz = tests.mockbackend.make_bz(
        user_login_args="data/mockargs/test_login.txt",
        user_login_return=RuntimeError("TEST ERROR"))
    out = run_cli(cmd, fakebz, expectfail=True)
    assert "Login failed: TEST ERROR" in out

    fakebz = tests.mockbackend.make_bz(
        user_login_args="data/mockargs/test_login.txt",
        user_login_return={})
    out = run_cli(cmd, fakebz)
    assert "Login successful" in out

    cmd = "bugzilla --restrict-login --user FOO --password BAR login"
    fakebz = tests.mockbackend.make_bz(
        user_login_args="data/mockargs/test_login-restrict.txt",
        user_login_return={})
    out = run_cli(cmd, fakebz)
    assert "Login successful" in out

    cmd = "bugzilla --ensure-logged-in --user FOO --password BAR login"
    # Raises raw error trying to see if we aren't logged in
    with pytest.raises(NotImplementedError):
        fakebz = tests.mockbackend.make_bz(
            user_login_args="data/mockargs/test_login.txt",
            user_login_return={},
            user_get_args=None,
            user_get_return=NotImplementedError())
        out = run_cli(cmd, fakebz)

    # Errors with expected code
    cmd = "bugzilla --ensure-logged-in --user FOO --password BAR login"
    fakebz = tests.mockbackend.make_bz(
        user_login_args="data/mockargs/test_login.txt",
        user_login_return={},
        user_get_args=None,
        user_get_return=bugzilla.BugzillaError("TESTMESSAGE", code=505))
    out = run_cli(cmd, fakebz, expectfail=True)
    assert "--ensure-logged-in passed but you" in out

    # Returns success for logged_in check and hits a tokenfile line
    cmd = "bugzilla --ensure-logged-in "
    cmd += "login FOO BAR"
    tmp = tempfile.NamedTemporaryFile()
    fakebz = tests.mockbackend.make_bz(
        bz_kwargs={"use_creds": True, "tokenfile": tmp.name},
        user_login_args="data/mockargs/test_login.txt",
        user_login_return={'id': 1234, 'token': 'my-fake-token'},
        user_get_args=None,
        user_get_return={})
    fakebz.connect("https://example.com")
    out = run_cli(cmd, fakebz)
    assert "Token cache saved" in out
    assert fakebz.tokenfile in out
    assert "Consider using bugzilla API" in out
    tests.utils.diff_compare(open(tmp.name).read(),
            "data/clioutput/tokenfile.txt")

    # Returns success for logged_in check and hits another tokenfile line
    cmd = "bugzilla --ensure-logged-in "
    cmd += "login FOO BAR"
    fakebz = tests.mockbackend.make_bz(
        bz_kwargs={"use_creds": True, "tokenfile": None},
        user_login_args="data/mockargs/test_login.txt",
        user_login_return={'id': 1234, 'token': 'my-fake-token'},
        user_get_args=None,
        user_get_return={})
    out = run_cli(cmd, fakebz)
    assert "Token not saved" in out


def test_interactive_login(monkeypatch, run_cli):
    bz = tests.mockbackend.make_bz(
        user_login_args="data/mockargs/test_interactive_login.txt",
        user_login_return={},
        user_logout_args=None,
        user_logout_return={},
        user_get_args=None,
        user_get_return={})

    tests.utils.monkeypatch_getpass(monkeypatch)
    cmd = "bugzilla login"
    fakestdin = "fakeuser\nfakepass\n"
    out = run_cli(cmd, bz, stdin=fakestdin)
    assert "Bugzilla Username:" in out
    assert "Bugzilla Password:" in out

    # API key prompting and saving
    tmp = tempfile.NamedTemporaryFile()
    bz.configpath = [tmp.name]
    bz.url = "https://example.com"

    cmd = "bugzilla login --api-key"
    fakestdin = "MY-FAKE-KEY\n"
    out = run_cli(cmd, bz, stdin=fakestdin)
    assert "API Key:" in out
    assert tmp.name in out
    tests.utils.diff_compare(open(tmp.name).read(),
            "data/clioutput/test_interactive_login_apikey_rcfile.txt")

    # Check that we don't attempt to log in if API key is configured
    assert bz.api_key
    cmd = "bugzilla login"
    out = run_cli(cmd, bz)
    assert "already have an API" in out



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

# This class needs to live in rhbugzilla.py to preserve historical
# 'bugzilla.rhbugzilla' import compat

from .oldclasses import RHBugzilla  # pylint: disable=unused-import



from setuptools import setup, find_packages
import pathlib

from distutils.core import Command
from unittest import TextTestRunner, TestLoader
import os

bugzilla = []
for x in os.listdir('bugzilla/'):
    bugzilla.append('bugzilla/%s' % x)

setup(name='ksc',
      version='1.12',
      description="ksc tool",
      long_description="Kernel Module Source Checker tool",
      url='https://github.com/RedHatOfficial/ksc/',
      platforms=["Linux"],
      classifiers= [
          'Development Status :: 5 - Production/Stable',
          'Intended Audience :: Developers',
          'Topic :: Software Development :: Build Tools',
          'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
          'Programming Language :: Python :: 3'
      ],
      python_requires='>=3.6, <4',
      install_requires=[
          'requests'
      ],
      author="Kushal Das, Samikshan Bairagya, Stanislav Kozina, Martin Lacko, Ziqian Sun",
      author_email="kdas@redhat.com, sbairagy@redhat.com, skozina@redhat.com, mlacko@redhat.com, zsun@redhat.com",
      license="http://www.gnu.org/copyleft/gpl.html",
      data_files=[("/usr/bin", ['ksc']),
                  ('/etc', ['data/ksc.conf']),
                  ('/usr/share/ksc', ['ksc.py', 'utils.py']),
                  ('/usr/share/ksc/data', ['data/ksc.conf']),
                  ('/usr/share/ksc/bugzilla', bugzilla)])



#!/usr/bin/env python
#
# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

# apikey.py: Demostrate prompting for API key and passing it to Bugzilla
# pylint: disable=undefined-variable

import bugzilla

# Don't worry, changing things here is fine, and won't send any email to
# users or anything. It's what landfill.bugzilla.org is for!
URL = "https://landfill.bugzilla.org/bugzilla-5.0-branch/xmlrpc.cgi"

print("You can get an API key at:\n "
      "    https://landfill.bugzilla.org/bugzilla-5.0-branch/userprefs.cgi")
print("This is a test site, so no harm will come!\n")

api_key = input("Enter Bugzilla API Key: ")

# API key usage assumes the API caller is storing the API key; if you would
# like to use one of the login options that stores credentials on-disk for
# command-line usage, use login tokens.
bzapi = bugzilla.Bugzilla(URL, api_key=api_key)
assert bzapi.logged_in



# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

import os


class _CLICONFIG(object):
    def __init__(self):
        self.REDHAT_URL = None
        self.REGENERATE_OUTPUT = False
        self.ONLY_REST = False
        self.ONLY_XMLRPC = False


CLICONFIG = _CLICONFIG()
os.environ["__BUGZILLA_UNITTEST"] = "1"



#
# Copyright Red Hat, Inc. 2012
#
# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.
#

"""
Test miscellaneous API bits
"""

import pytest

import bugzilla

import tests
import tests.mockbackend


def test_mock_rhbz():
    fakebz = tests.mockbackend.make_bz(rhbz=True)
    assert fakebz.__class__ == bugzilla.RHBugzilla


def test_file_imports():
    # Ensure historically stable import paths continue to work
    # pylint: disable=unused-import
    from bugzilla.rhbugzilla import RHBugzilla
    from bugzilla.bug import Bug
    from bugzilla.base import Bugzilla


def testUserAgent():
    b3 = tests.mockbackend.make_bz(version="3.0.0")
    assert "python-bugzilla" in b3.user_agent


def test_fixurl():
    assert (bugzilla.Bugzilla.fix_url("example.com") ==
        "https://example.com/xmlrpc.cgi")
    assert (bugzilla.Bugzilla.fix_url("example.com", force_rest=True) ==
        "https://example.com/rest/")
    assert (bugzilla.Bugzilla.fix_url("example.com/xmlrpc.cgi") ==
        "https://example.com/xmlrpc.cgi")
    assert (bugzilla.Bugzilla.fix_url("http://example.com/somepath.cgi") ==
        "http://example.com/somepath.cgi")


def testPostTranslation():
    def _testPostCompare(bz, indict, outexpect):
        outdict = indict.copy()
        bz.post_translation({}, outdict)
        assert outdict == outexpect

        # Make sure multiple calls don't change anything
        bz.post_translation({}, outdict)
        assert outdict == outexpect

    bug3 = tests.mockbackend.make_bz(version="3.4.0")
    rhbz = tests.mockbackend.make_bz(version="4.4.0", rhbz=True)

    test1 = {
        "component": ["comp1"],
        "version": ["ver1", "ver2"],

        'flags': [{
            'is_active': 1,
            'name': 'qe_test_coverage',
            'setter': 'pm-rhel@redhat.com',
            'status': '?',
        }, {
            'is_active': 1,
            'name': 'rhel-6.4.0',
            'setter': 'pm-rhel@redhat.com',
            'status': '+',
        }],

        'alias': ["FOO", "BAR"],
        'blocks': [782183, 840699, 923128],
        'keywords': ['Security'],
        'groups': ['redhat'],
    }

    out_simple = test1.copy()
    out_simple["components"] = out_simple["component"]
    out_simple["component"] = out_simple["components"][0]
    out_simple["versions"] = out_simple["version"]
    out_simple["version"] = out_simple["versions"][0]

    _testPostCompare(bug3, test1, test1)
    _testPostCompare(rhbz, test1, out_simple)


def test_rhbz_pre_translation():
    bz = tests.mockbackend.make_bz(rhbz=True)
    input_query = {
        "bug_id": "12345,6789",
        "component": "comp1,comp2",
        "column_list": ["field1", "field8"],
    }

    bz.pre_translation(input_query)
    output_query = {
        'component': ['comp1', 'comp2'],
        'id': ['12345', '6789'],
        'include_fields': ['field1', 'field8', 'id'],
    }

    assert output_query == input_query


def testUpdateFailures():
    # sub_component without component also passed
    bz = tests.mockbackend.make_bz(version="4.4.0", rhbz=True)
    with pytest.raises(ValueError):
        bz.build_update(sub_component="some sub component")

    # Trying to update value that only rhbz supports
    bz = tests.mockbackend.make_bz()
    with pytest.raises(ValueError):
        bz.build_update(fixed_in="some fixedin value")


def testCreatebugFieldConversion():
    bz4 = tests.mockbackend.make_bz(version="4.0.0")
    vc = bz4._validate_createbug  # pylint: disable=protected-access
    out = vc(product="foo", component="bar",
        version="12", description="foo", short_desc="bar",
        check_args=False)
    assert out == {
        'component': 'bar', 'description': 'foo', 'product': 'foo',
        'summary': 'bar', 'version': '12'}


def testURLSavedSearch():
    bz4 = tests.mockbackend.make_bz(version="4.0.0")
    url = ("https://bugzilla.redhat.com/buglist.cgi?"
        "cmdtype=dorem&list_id=2342312&namedcmd="
        "RHEL7%20new%20assigned%20virt-maint&remaction=run&"
        "sharer_id=321167")
    query = {
        'sharer_id': '321167',
        'savedsearch': 'RHEL7 new assigned virt-maint'
    }
    assert bz4.url_to_query(url) == query


def testStandardQuery():
    bz4 = tests.mockbackend.make_bz(version="4.0.0")
    url = ("https://bugzilla.redhat.com/buglist.cgi?"
        "component=virt-manager&query_format=advanced&classification="
        "Fedora&product=Fedora&bug_status=NEW&bug_status=ASSIGNED&"
        "bug_status=MODIFIED&bug_status=ON_DEV&bug_status=ON_QA&"
        "bug_status=VERIFIED&bug_status=FAILS_QA&bug_status="
        "RELEASE_PENDING&bug_status=POST&order=bug_status%2Cbug_id")
    query = {
        'product': 'Fedora',
        'query_format': 'advanced',
        'bug_status': ['NEW', 'ASSIGNED', 'MODIFIED', 'ON_DEV',
            'ON_QA', 'VERIFIED', 'FAILS_QA', 'RELEASE_PENDING', 'POST'],
        'classification': 'Fedora',
        'component': 'virt-manager',
        'order': 'bug_status,bug_id'
    }
    assert bz4.url_to_query(url) == query

    # Test with unknown URL
    assert bz4.url_to_query("https://example.com") == {}


def test_api_login():
    with pytest.raises(TypeError):
        # Missing explicit URL
        bugzilla.Bugzilla()

    with pytest.raises(Exception):
        # Calling connect() with passed in URL
        bugzilla.Bugzilla(url="https:///FAKEURL")

    bz = tests.mockbackend.make_bz()

    with pytest.raises(ValueError):
        # Errors on missing user
        bz.login()

    bz.user = "FOO"
    with pytest.raises(ValueError):
        # Errors on missing pass
        bz.login()

    bz.password = "BAR"
    bz.api_key = "WIBBLE"
    with pytest.raises(ValueError):
        # Errors on api_key + login()
        bz.login()

    # Hit default api_key code path
    bz = tests.mockbackend.make_bz(
        bz_kwargs={"api_key": "FAKE_KEY"},
        user_login_args="data/mockargs/test_api_login.txt",
        user_login_return={})
    # Try reconnect, with RHBZ testing
    bz.connect("https:///fake/bugzilla.redhat.com")
    bz.connect()

    # Test auto login if user/password is set
    bz = tests.mockbackend.make_bz(
        bz_kwargs={"user": "FOO", "password": "BAR"},
        user_login_args="data/mockargs/test_api_login2.txt",
        user_login_return={},
        user_logout_args=None,
        user_logout_return={})

    # Test logout
    bz.logout()


def test_version_bad():
    # Hit version error handling
    bz = tests.mockbackend.make_bz(version="badversion")
    assert bz.bz_ver_major == 5
    assert bz.bz_ver_minor == 0

    # pylint: disable=protected-access
    assert bz._get_version() == 5.0


def test_extensions_bad():
    # Hit bad extensions error handling
    tests.mockbackend.make_bz(extensions="BADEXTENSIONS")


def test_bad_scheme():
    bz = tests.mockbackend.make_bz()
    try:
        bz.connect("ftp://example.com")
    except Exception as e:
        assert "Invalid URL scheme: ftp" in str(e)


def test_update_flags():
    # update_flags is just a compat wrapper for update_bugs
    bz = tests.mockbackend.make_bz(
        bug_update_args="data/mockargs/test_update_flags.txt",
        bug_update_return={})
    bz.update_flags([12345, 6789], {"name": "needinfo", "status": "?"})


def test_bugs_history_raw():
    # Stub test for bugs_history_raw
    ids = ["12345", 567]
    bz = tests.mockbackend.make_bz(
        bug_history_args=(ids, {}),
        bug_history_return={})
    bz.bugs_history_raw(ids)


def test_get_comments():
    # Stub test for get_commands
    ids = ["12345", 567]
    bz = tests.mockbackend.make_bz(
        bug_comments_args=(ids, {}),
        bug_comments_return={})
    bz.get_comments(ids)


def test_get_xmlrpc_proxy():
    # Ensure _proxy goes to a backend API
    bz = tests.mockbackend.make_bz()
    with pytest.raises(NotImplementedError):
        dummy = bz._proxy  # pylint: disable=protected-access

    assert bz.is_xmlrpc() is False
    assert bz.is_rest() is False
    assert hasattr(bz.get_requests_session(), "request")


def test_requests_session_passthrough():
    import requests
    session = requests.Session()

    bz = tests.mockbackend.make_bz(
        bz_kwargs={"requests_session": session, "sslverify": False})
    assert bz.get_requests_session() is session
    assert session.verify is False


def test_query_url_fail():
    # test some handling of query from_url errors
    query = {"query_format": "advanced", "product": "FOO"}
    checkstr = "does not appear to support"

    exc = bugzilla.BugzillaError("FAKEERROR query_format", code=123)
    bz = tests.mockbackend.make_bz(version="4.0.0",
        bug_search_args=None, bug_search_return=exc)
    try:
        bz.query(query)
    except Exception as e:
        assert checkstr in str(e)

    bz = tests.mockbackend.make_bz(version="5.1.0",
        bug_search_args=None, bug_search_return=exc)
    try:
        bz.query(query)
    except Exception as e:
        assert checkstr not in str(e)
